__doc__ = """CARSDoc product module."""
__version__ = '0.1'

import string
from Globals import HTMLFile,MessageDialog,Persistent
import OFS.Folder
import OFS.Image
import Acquisition
import AccessControl
from OFS.content_types import guess_content_type
from Products.CARS.PropertyManager import PropertyClient, PropertyStore
manage_addCARSDocForm = HTMLFile('CARSdocAdd', globals()) 
from Products.CARS import Log
from Products.CARS.CARSFolder import CARSFolder
from Products.CARS.CARSObject import CARSObject
from Products.CARS.objBase import objBase

def manage_addCARSDoc(self, id, title='', revisioNum=1, REQUEST=None):
    """Add a CARSDoc to a folder."""
    doc=CARSDoc(id, title=title, revisioNum=revisioNum)
    self._setObject(id, doc)
    doc=self._getOb(id)
    Log.carslog(self, 'addCARSDoc(%s)' % repr(id), REQUEST=REQUEST)
    if REQUEST is not None:
        doc.informContacts("Doc Added", "Document added: %s" % doc.id)
        return self.manage_main(self, REQUEST)

class CARSDoc(CARSObject.CARSeXternFS): 
    
    __dbgLevelDiagnostic = 0
    __className='CARSDocument'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    meta_type = 'CARS Document'
    __ac_permissions__=CARSObject.CARSObject.__ac_permissions__

    pages=[]
    nextpage=1
    images=None
    ocr=None
    smallpages=None
    dirty=[]
    originalSubPath='original'
    imageSubPath='images'
    ocrSubPath='ocr'

    def __init__(self, id, title='', revisioNum=1): 
        CARSDoc.inheritedAttribute("__init__") (self, id, title, revisioNum)
        self.pages=list(())
        self.dirty=list(())
        self.imageSubPath='images'
        self.ocrSubPath='ocr'
        self.originalSubPath='original'
        self.originalFilename='original'
        self.nextpage=1

    index_html = HTMLFile('index', globals()) # View Interface
    manage_main = HTMLFile('index', globals()) # View Interface
    manage_edit = HTMLFile('CARSdocEdit', globals()) # Management Interface
    showPageForm = HTMLFile('viewPage', globals())

    def _sane_page(self, page):
        if page is None:
            page = self.nextpage
        try: page = int(page)
        except: raise 'BadRequest', 'Invalid page number: %s, integer expected.' % page
        if not (page > 0): raise 'BadRequest', 'Invalid page number: %d' % page
        return page

    def _get_content_type(self, file, id='', content_type=None):
        headers=getattr(file, 'headers', None)
        self.__dbg(200, 'headers %s' % (headers))
        if headers and headers.has_key('content-type'):
            content_type=headers['content-type']
            self.__dbg(200, "found content-type: %s" % (content_type))
        else:
            content_type, enc=guess_content_type(
                getattr(file, 'filename',id), file, content_type)
        return content_type

    def manage_addPage(self, title='', image='', ocr='', page=None, REQUEST=None):
        """Add a page to the CARS document"""
        self.__dbg(0, "manage_addPage: %s" % (page))
        self.testImmutable()
        page=self._sane_page(page)
        imageSP=self.imageSubPath
        ocrSP=self.ocrSubPath
        if page in self.pages:
            raise 'BadRequest', 'Duplicate page number: %d' % page
        try:
            FSpath=self.absolute_url(1)
        except:
            FSpath='cars/store'
        content_type=self._get_content_type(image)
        self.__dbg(200, 'image contentType: %s' % content_type)
        self.appendeXternFS(image, str(page), title=title, \
                            content_type=content_type, \
                            subPath=imageSP)
        content_type=self._get_content_type(ocr)
        self.appendeXternFS(ocr, str(page), content_type=content_type, \
                            subPath=ocrSP)
        self.pages.append(page)
        self._p_changed=1
        if page is not self.nextpage:
            self.pages.sort()
        self.nextpage=(self.pages[-1])+1
        Log.carslog(self, 'addPage(%s)' % repr(page), REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def manage_attachDoc(self, original='', REQUEST=None):
        """Attach original document"""
        self.testImmutable()
        self.__dbg(200, "entered")
        try:
            original.filename=string.split(string.replace(original.filename, " ", "_"), '\\')[-1]
            original.filename=string.split(string.replace(original.filename, "/", "-"), '\\')[-1]
            origFn=original.filename
        except:
            origFn='attached.pdf'
        self.__dbg(200, "original filename= %s" % origFn)

        origSP=self.originalSubPath
        if origFn=='':
            return
        try:
            FSpath=self.absolute_url(1)
        except:
            FSpath='cars/store'
        self.deleteSubPathFS(origSP)
        content_type=self._get_content_type(original)
        self.__dbg(200, 'doc contentType: %s' % content_type)
#        content_type=self._get_content_type(origFn)
        self.appendeXternFS(original, origFn, title=self.title, \
                            content_type=content_type, \
                            subPath=origSP)
        Log.carslog(self, 'addFile(%s)' % origFn, REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def manage_removeAttachedDoc(self, REQUEST=None):
        """Remove original attached documents if any"""
        self.testImmutable()
        self.deleteSubPathFS(self.originalSubPath)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')        

    def getAttachedDoc(self, REQUEST=None):
        return self.geteXternFSDetails(self.originalSubPath)
        r=[]
        fn=self.geteXternFilenameList(self.originalSubPath)
        if len(fn)>0:
            fn=fn[0]
            size=self._getSize(fn)
        else:
            fn=''
            size='None'
        r.append({'filename': fn, 'size': size})
        self.__dbg(200, "returning : %s" % (r))
        return r
    
    def getAttachedDocLink(self, filename, REQUEST=None):
        """GetFileLink"""
        self.REQUESTeXternFS(filename, self.originalSubPath, REQUEST)

    def manage_delPages(self, pages=[], REQUEST=None):
        """Delete Pages"""
        self.testImmutable()
        for p in pages:
            self.manage_delPage(p, REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def pageTitle(self, page):
        """Return the title of a page"""
        page=self.validatePage(page)
        return self.geteXternFShandle(str(page), subPath=self.imageSubPath).title

    def manage_delPage(self, page, REQUEST=None):
        """Delete a page from the CARS document"""
        page=self.validatePage(page)
        self.removeXternFS(str(page), subPath=self.imageSubPath)
        self.removeXternFS(str(page), subPath=self.ocrSubPath)
        self.pages.remove(page)
        self._p_changed=1
        self.mark_dirty(page)
        if self.pages == []:
            self.nextpage=1
        else:
            self.nextpage=(self.pages[-1])+1
        Log.carslog(self, 'deletePage(%s)' % repr(page), REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def manage_updatePage(self, page, title='', image='', ocr='', REQUEST=None):
        """Update a page of the CARS document"""
        self.testImmutable()
        page=self.validatePage(page)
        pimage=self.images._getOb(str(page))
        pocr=self.ocr._getOb(str(page))
        if image:
            pimage.manage_upload(file=image)
        if ocr:
            pocr.manage_upload(file=ocr)
        if title:
            setattr(pimage, 'title', title)
            setattr(pocr, 'title', title)
        self.mark_dirty(page)
        Log.carslog(self, 'updatePage(%s)' % repr(page), REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def mark_dirty(self, page):
        if page not in self.dirty:
            self.dirty.append(page)
            self._p_changed=1

    def is_dirty(self, page):
        return page in self.dirty

    def mark_clean(self, page):
        if page in self.dirty:
            self.dirty.remove(page)
            self._p_changed=1

    def validatePage(self, page):
        page = self._sane_page(page)
        if not (page in self.pages):
            raise 'BadRequest', 'No such page: %s' % page
        return page

    def getPageData(self, page):
        self.validatePage(page)
        ob=self.images._getOb(str(page))
        return ob.data

    def getPage(self, page, REQUEST=None):
        """x"""
        page=self.validatePage(page)
        ob=self.geteXternFShandle(str(page), subPath=self.imageSubPath)
        outPage=self.geteXternFS(str(page), subPath=self.imageSubPath)
        Log.carslog(self, 'showPage(%s)' % repr(page), REQUEST=REQUEST)
        if REQUEST:
            REQUEST['RESPONSE'].setBody(outPage)
            REQUEST['RESPONSE'].setHeader('Content-Type', ob.content_type)
        return outPage

    def showPage(self, page, REQUEST=None):
        """Show a page to the user"""
        self.validatePage(page)
        page=int(page)
        if REQUEST:
            REQUEST['RESPONSE']=self.showPageForm(self, page=page)
        return self.showPageForm(self, page=page)

    def next_page(self, page, wrap=0):
        self.validatePage(page)
        i=self.pages.index(page)+1
        if i == len(self.pages):
            if wrap: return self.pages[0]
            else: raise 'BadRequest', '%s is already the last page.' % page
        else: return self.pages[i]

    def prev_page(self, page, wrap=0):
        self.validatePage(page)
        i=self.pages.index(page)
        if not (i or wrap):
            raise 'BadRequest', '%s is the first page.' % page
        else: return self.pages[i-1]

    manage_logout=CARSFolder.CARSFolder.manage_logout
