__doc__ = """CARS File Group product module."""
__version__ = '0.1'

import string, StringIO
from Globals import HTMLFile,MessageDialog,Persistent
import OFS.Folder
import OFS.Image
import Acquisition
import AccessControl
from Products.CARS.PropertyManager import PropertyClient, PropertyStore
from Products.CARS import Log
from Products.CARS.CARSFolder import CARSFolder
from Products.CARS.CARSObject import CARSObject
from Products.CARS.objBase import objBase
from urllib import quote

manage_addCARSFileGroupForm = HTMLFile('carsfileAdd', globals()) 

def manage_addCARSFileGroup(self, id, title='', revisioNum=1, REQUEST=None):
    """Add an CARS File Group to a folder."""
    fobj=CARSFileGroup(id, title=title, revisioNum=revisioNum)
    self._setObject(id, fobj)
    fobj=self._getOb(id)
    Log.carslog(self, 'addCARSFileGroup(%s)' % repr(id), REQUEST=REQUEST)
    if REQUEST is not None:
        fobj.informContacts("File Group Added", "CARS File Group added: %s" % fobj.id) 
        return self.manage_main(self, REQUEST)

class CARSFileGroup(CARSObject.CARSeXternFS): 
    
    __dbgLevelDiagnostic = 0
    __className='CARSFileGroup'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    meta_type = 'CARS File Group'
    __ac_permissions__=CARSObject.CARSObject.__ac_permissions__

    def __init__(self, id, title='', revisioNum=1): 
        CARSFileGroup.inheritedAttribute("__init__") (self, id, title, revisioNum)
        self.fileIdList=[]
        self.fileSubPath='files'
        self.nextfileId=1

    index_html = HTMLFile('index', globals()) # View Interface
    manage_main = HTMLFile('index', globals()) # View Interface
    manage_edit = HTMLFile('carsfileEdit', globals()) # Management Interface
    showFileForm = HTMLFile('viewFile', globals())
    showFileListForm = HTMLFile('viewFileList', globals())

    def manage_uploadFile(self, filename='', fileId='', REQUEST=None):
        """Upload a file"""
        file=filename
        self.testImmutable()
        if fileId=='':
            fileId=file.filename
        fileId=string.split(string.replace(fileId, " ", "_"), '\\')[-1]
        file.filename=string.split(string.replace(file.filename, " ", "_"), '\\')[-1]
        self.__dbg(1, "Filename %s, fildId: %s" % (file.filename, fileId))
        if fileId in self.fileIdList:
            raise 'BadRequest', 'Duplicate File Id: %s' % fileId
        FSpath=self.absolute_url(1)
        self.appendeXternFS(file, fileId, \
                            subPath=self.fileSubPath)
        Log.carslog(self, 'addFile(%s)' % repr(fileId), REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def fileList(self):
        """Return a list of filenames"""
        return self.geteXternFilenameList(subPath=self.fileSubPath)

    def fileTuple(self):
        """Return a list of tuples of filenames"""
        r=[]
        for eFile in self.geteXternFilenameList(subPath=self.fileSubPath):
            r.append({'filename':eFile})
        return r

    def manage_delFiles(self, fileIdList=[], REQUEST=None):
        """Delete Files"""
        self.testImmutable()
        for fileId in fileIdList:
            self.manage_delFileId(fileId, \
                                  REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def manage_delFileId(self, fileId, REQUEST=None):
        """Delete a file"""
        self.__dbg(1, "Removing fileid %s" %(fileId))
        self.removeXternFS(fileId, subPath=self.fileSubPath)
        Log.carslog(self, 'deleteFile(%s)' % repr(fileId), REQUEST=REQUEST)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url() + '/manage_edit')

    def anyFilesUploaded(self):
        return self.existFSList()

    def getAttachedFiles(self, REQUEST=None):
        return self.geteXternFSDetails(self.fileSubPath)
 
    def showFiles(self, fileIdList=[], REQUEST=None):
        """Show a file to the user"""
        def _tuple(eFile):
            return {'filename': eFile}

        files=fileIdList
        self.__dbg(1, "files selected: %s" % (files))
        self.__dbg(1, "viewfiles: %s" % (map(_tuple, files)))
        if REQUEST:	
	    REQUEST['RESPONSE']=self.showFileListForm(self, viewFiles=map(_tuple, files))
        return self.showFileListForm(self, viewFiles=map(_tuple, files))

    def getAttachedFileLink(self, filename, REQUEST=None):
        """GetFileLink"""
        self.REQUESTeXternFS(filename, self.fileSubPath, REQUEST=REQUEST)

    manage_logout=CARSFolder.CARSFolder.manage_logout

