__doc__ = """CARSFolder product module."""
__version__ = '0.1'
import string
from DateTime import DateTime
from Globals import HTMLFile,MessageDialog,Persistent
import OFS.Folder
import Acquisition
import AccessControl
from AccessControl import getSecurityManager
from ZPublisher.Converters import get_converter
from Products.CARS.PropertyManager import PropertyServer,PropertyClient,PropertyStore
from Products.CARS import Log
from Products.CARS.objBase import objBase
from Products.CARS.kwLists import kwLists

manage_addCARSFolderForm = HTMLFile('carsfolderAdd', globals())

def manage_addCARSFolder(self, id, title='', revisioNum=1, REQUEST=None):
    """Add an CARS Folder to a folder."""
    Log.carslog(self, 'addCARSFolder(%s)' % repr(id), REQUEST=REQUEST)
    ob=CARSFolder()
    ob.id=id
    ob.title=title
    ob.revisioNum=revisioNum
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

def SimpleFindProperty(ids, props, searchterm, path='', all=0):
    def dbg(str):
        f=open('/tmp/zopeslog', 'a', 0)
        f.write('%s\n' % str)
        f.close()
    def findInTokens(term, tokens):
        return term in tokens
    def findInString(term, str):
        return (string.find(str, term) != -1)
    def findInDate(term, date):
        try: dt=DateTime(term)
        except DateTime.SyntaxError: return 0
        return ((dt - date) == 0)
    find_table = {'tokens': findInTokens,
                  'string': findInString,
                  'date': findInDate}
    result=[]
    checkPermission=getSecurityManager().checkPermission
    for ob in ids:
        if hasattr(ob,'propertyMap') and checkPermission('Find Properties',ob):
            for entry in ob.propertyMap():
                id=entry['id']
                propType=entry['type']
                if (all or (id in props)):
                    f=find_table.get(propType, None)
                    if f and f(searchterm, ob.getProperty(id)):
                        result.append({'object': ob,
                                       'id': path + ob.id,
                                       'url': ob.absolute_url()})
        if getattr(ob, 'isPrincipiaFolderish', None):
            result.extend(findProperty(ob.objectValues(), props, searchterm, \
                                       path=path+ob.id+'.', all=all))
    return result

def every(fun, lis):
    for i in lis:
        if not fun(i): return 0
    return 1
def any(fun, lis):
    for i in lis:
        if fun(i): return 1
    return 0

def findProperty(objects, patterns, path='', all=1, container=None):
    """Find a property
    """
    def dbg(str):
        f=open('/tmp/zopeslog', 'a', 0)
        f.write('%s\n' % str)
        f.close()
    def apply_case(spec, str):
        case=spec['case-sensitive']
        if not case: return string.lower(str)
        else: return str
    def string_matcher(spec, dbg=dbg):
        def exact(str, p):
            return (str == p)
        def inexact(str, p, dbg=dbg):
            return ((string.find(str,p)) != -1)
        if spec['exact']:
            return exact
        else:
            return inexact
    def matchTokens(spec, tokens, dbg=dbg, apply_case=apply_case, string_matcher=string_matcher, container=container):
        all=spec['all']
        words=spec['items']
        tokens = map(lambda x, apply_case=apply_case, spec=spec:
                     apply_case(spec, x),
                     tokens)
        words = map(lambda x, apply_case=apply_case, spec=spec:
                     apply_case(spec, x),
                     words)
        sm=string_matcher(spec)
        def m(x, sm=sm, tokens=tokens, dbg=dbg):
            dbg('matchTokens: matching %s with %s' % (x, tokens))
            return any(lambda t, sm=sm, x=x: sm(t, x), tokens)
        if all: return every(m, words)
        else: return any(m, words)
    def matchString(spec, str, apply_case=apply_case, string_matcher=string_matcher):
        pattern=spec['pattern']
        pattern=apply_case(spec, pattern)
	str=apply_case(spec,str)
        m=string_matcher(spec)
        return m(str, pattern)
    def matchDate(spec, date):
        start=spec['start']
        end=spec['end']
        return (date.greaterThanEqualTo(start) and date.lessThanEqualTo(end))
    def matchBoolean(spec, bool):
        return (not spec['on']) == (not bool)
    def matchNumber(spec, num):
        n=spec['value']
        op=spec['operator']
        if op == 'eq':
            return num==n
        elif op == 'ne':
            return num!=n
        elif op == 'lt':
            return num < n
        elif op == 'le':
            return num <= n
        elif op == 'gt':
            return num > n
        elif op == 'ge':
            return num >= n
        else: return 0
    
    find_table = {'tokens': matchTokens,
                  'string': matchString,
                  'lines': matchTokens,
                  'meme': matchTokens,
                  'date': matchDate,
                  'text': matchString,
                  'boolean': matchBoolean,
                  'long': matchNumber,
                  'int': matchNumber,
                  'float': matchNumber}
    
    def match_prop(ob, p, path=path, find_table=find_table, dbg=dbg):
        id=p['id']
        if not ob.hasProperty(id): return 0
        propType=ob.propertyType(id)
        dbg('matching %s, %s (%s) on %s with %s' % (propType, id, ob.getProperty(id), path + ob.id, p))
        ## match properties of the same type, else do not perform checking
        if p['type'] == propType:
            f=find_table.get(propType, None)
            return (f and f(p, ob.getProperty(id)))
    
    result=[]
    checkPermission=getSecurityManager().checkPermission
    for ob in objects:
        if hasattr(ob,'propertyMap') and checkPermission('Find Properties',ob):
            dbg('Looking in %s' % path + ob.id)
            if (all and every(lambda p, match_prop=match_prop, ob=ob: match_prop(ob, p), patterns)) or \
               (all==0 and any(lambda p, match_prop=match_prop, ob=ob: match_prop(ob, p), patterns)):
                result.append({'object': ob,
                               'id': path + ob.id,
                               'url': ob.absolute_url()})
        if getattr(ob, 'isPrincipiaFolderish', None):
            result.extend(findProperty(ob.objectValues(), patterns, \
                                       path=path+ob.id+'.', all=all, container=ob))
    return result


class CARSFolder(PropertyServer.PropertyServer,
                PropertyClient.PropertyClient,
                PropertyStore.PropertyStore,
                kwLists.kwLists,
                objBase.objBase,
                OFS.Folder.Folder,
                Persistent,
                Acquisition.Implicit,
                AccessControl.Role.RoleManager,
                ):

    meta_type = 'CARS Folder'
    is_property_manager = 1
    __dbgLevelDiagnostic=0
    __className='CARSFolder'
    __oldPath=''

    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    __ac_permissions__=(
        ('Access contents information',
         ('objectIds', 'objectValues', 'objectItems', 'subObjects', '',)),
        ('Find Properties',
         ('manage_findPropertyForm', 'manage_findProperty',)),
        ('View',
         ('index_html', 'manage', 'manage_menu', 'manage_main', 'manage_workspace','')),
        ('Delete objects',
         ('manage_delObjects',)),
        ('Rename objects',
         ('manage_renameObjects', 'manage_renameForm')),
        ('Copy/Paste',
         ('manage_cutObjects', 'manage_copyObjects', 'manage_pasteObjects')),
        ('Manage properties',
         ('fullPropertyMap', 'manage_propertiesForm', 'manage_editProperties',
	  'manage_refObjectForm')),
        ('Manage property map',
         ('manage_definePropertiesForm', 'manage_delPropertyDefs',
          'manage_editPropertyMap', 'manage_addPropertyDef')),
        ('Logout', ('manage_logout',)))

    manage_options = (
        {'label': 'Folder View', 'action': 'manage_main',
         'image': 'folder-view'},
        {'label': 'Properties', 'action': 'manage_propertiesForm',
         'image': 'properties'},
        {'label': 'Define Properties', 'action': 'manage_definePropertiesForm',
         'image': 'define-properties'},
        {'label': 'Search', 'action': 'manage_findPropertyForm',
	 'image': 'search'},
        {'label': 'Log out', 'action': 'manage_logout', 'image': 'logout'},)

    index_html = HTMLFile('manage', globals())
    manage_main = HTMLFile('index', globals())
    manage_menu = HTMLFile('menu', globals())

    manage_findPropertyForm=HTMLFile('findProperty', globals())
    findPropertyResult=HTMLFile('findPropertyResult', globals())
    manage_propertiesForm=HTMLFile('../PropertyManager/properties', globals())
    manage_definePropertiesForm=HTMLFile('../PropertyManager/defineProperties', globals())
    manage_refObjectForm=HTMLFile('referenceObject', globals())

    manage_UndoForm=HTMLFile('undo', globals())

    def __init__(self):
        OFS.Folder.Folder(self)
        CARSFolder.inheritedAttribute("__init__") (self)
        self.initKwVars()
        self.__dbg(1, "Initialised folder")

    def getFolderList(self, size=40, id=None, REQUEST=None):
        ## return on request a list of: folders, objects or files for the current container.
        ## type==all is every object in the container
        ## type==folder is every other container within the current
        ## type==objects every other object
        def getObjs(self, folder, propType):
            r=[]
            cp=getSecurityManager().checkPermission
            self.__dbg(1, "in folder: %s, %s" %(folder.id, folder))
            for ob in folder.objectValues():
                self.__dbg( 1, "found: %s, %s" %(ob.id, ob))
                if cp('Access contents information', ob):
                    url=string.replace(ob.getURL(), '%20', ' ')
                    self.__dbg(1, "ob: %s, url: %s/%s" %(ob.id, url, ob.id))
                    if (propType=='all') or \
                       (ob.meta_type in ('CARS Share', 'CARS Folder')
                        and (propType=='folder')):
                        label="[%s: '%s' %s]"%(ob.id, ob.title, ob.revisioNum)
                        r.append({'url': url+'/'+ob.id, 'label': label})
                    if (propType=='object') and \
                       not (ob.meta_type in ('CARS Share', 'CARS Folder')):
                        label="%s: '%s' %s"%(ob.id, ob.title, ob.revisioNum)
                        r.append({'url': url+'/'+ob.id, 'label': label})
            return r

        selectedOb=self.getProperty(id)
        folder=self
        if selectedOb:
            folder=selectedOb
            self.__dbg(1, "selected ob: %s, %s" %(folder.id, folder))
            if not (folder.meta_type in ('CARS Share', 'CARS Folder')) and \
               hasattr(folder, 'aq_parent'):
                if hasattr(folder.aq_parent, 'subObjects'):
                    folder=folder.aq_parent
                else:
                    folder=self
        f=getObjs(self, folder, 'folder')
        o=getObjs(self, folder, 'object')
        return f+o

    def filtered_objectIds(self):
        map(lambda x: x.id, filter(lambda x: getSecurityManager().checkPermission('Read Access', x), self.objectValues()))

    def propertyMap(self):
        """x"""
        def find_property(map, id):
            for md in map:
                if md['id'] == id:
                    return md
            return None
        def merge_maps(map1, map2, find_property=find_property):
            for md in map2:
                if not find_property(map1, md['id']):
                    map1.append(md)
        r = list(self._property_map)
        obj=self.aq_parent
        while 1:
            if hasattr(obj, 'meta_type'):
                if obj.meta_type in ('CARS Share', 'CARS Folder'):
                    merge_maps(r, obj.propertyMap())
                    break
            else:
                self.__dbg(5, "Obj: %s, has not meta_type" % (obj))
            if hasattr(obj, 'aq_parent'):
                obj=obj.aq_parent
            else: break
        return tuple(r)

    def containerPropertyMap(self):
        """return original map"""
        return list(self._property_map)

    def subObjects(self, REQUEST=None):
        """x"""
        r=[self.id]
        cp=getSecurityManager().checkPermission
        for ob in self.objectValues():
            if cp('Access contents information', ob):
                if getattr(ob, 'isPrincipiaFolderish', None) and \
                   hasattr(ob.aq_base, 'subObjects'):
                    r.append(ob.subObjects())
                else:
                    r.append(ob.id)
        return r

    def folderList(self, REQUEST=None):
	"""Return list of folders"""
	r=[]
        cp=getSecurityManager().checkPermission
	for ob in self.objectValues():
          if cp('Access contents information', ob):
	    if getattr(ob, 'isPrincipiaFolderish', None):
		r.append(ob.id)
	return 

    def changeFolder(self, folderID=None, REQUEST=None):
        """Change to new folder"""
	if not folderID:	
		return
	return 

    def selectObject(self, objectRef=None, REQUEST=None):
	"""Select the current object reference"""
        self.__dbg(1, REQUEST.get('PROPERTY', ''))
	if not objectRef:
		return
	self.__dbg(1, self.id)
	self.__dbg(1, self.getReqProp())
	return self.manage_refObjectForm(self, REQUEST)

    def manage_setRef(self):
	return folderList

    def dump(self):
        """Should only be called for debugging purposes only.
        Executing CARS startup from a shell, then closing the shell
        while having code dump to the screen will produce an IOError.
        """
        print self.propertyMap()
        print self.id
        print self

    def hasObject(self, id, REQUEST=None):
        """return true if the folder contains an object with id <id>"""
        _marker = []
        return self._getOb(id, default=_marker) is not _marker

    def manage_logout(self, REQUEST=None):
        """log out."""
        raise 'Unauthorized', 'Logged out.'

    def manage_simpleFindProperty(self, searchterm, props=[], allprops='all'):
        def dbg(str):
            f=open('/tmp/zopeslog', 'a', 0)
            f.write('%s\n' % str)
            f.close()
        def process_result(result):
            d={}
            for i in result:
                id=i['id']
                if not d.has_key(id):
                    d[id]=i
            values=d.values()
            values.sort(lambda x,y: cmp(x['id'], y['id']))
            return values
        
        found=findProperty(self.objectValues(), props,
                           searchterm, all=(allprops=='all'))
        return self.findPropertyResult(self, result=process_result(found),
                                       URL=self.absolute_url())

    def manage_findProperty(self, all=1, allshares=0, REQUEST=None):
        """Find a property."""
        Log.carslog(self,
                    'findProperty(all=%s, allshares=%s)' % (repr(all),
                                                            repr(allshares)),
                    REQUEST=REQUEST)
        def dbg(str):
            f=open('/tmp/zopeslog', 'a', 0)
            f.write('%s\n' % str)
            f.close()
        def process_result(result):
            d={}
            for i in result:
                id=i['id']
                if not d.has_key(id):
                    d[id]=i
            values=d.values()
            values.sort(lambda x,y: cmp(x['id'], y['id']))
            return values
        def field_value(id, field, propType=None, REQUEST=REQUEST, dbg=dbg):
            value=REQUEST.get('%s/%s' % (id, field))
            if propType:
                c=get_converter(propType, lambda x: x)
                value=c(value)
            return value
        def get_date_pattern(id, field_value=field_value):
            propType=field_value(id, 'type')
            if propType != 'date': raise 'BadRequest', 'Not a date field: %s' % id
            start=DateTime(field_value(id, 'start/year', 'int'),
                           field_value(id, 'start/month', 'int'),
                           field_value(id, 'start/day', 'int'),
			   0,0,0)
            end=DateTime(field_value(id, 'end/year', 'int'),
                         field_value(id, 'end/month', 'int'),
                         field_value(id, 'end/day', 'int'),
			 23,59,59)
            return {'id': id, 'type': 'date', 'start': start, 'end': end}
        def get_some_pattern(id, ttype, fields, field_value=field_value):
            propType=field_value(id, 'type')
            if propType != ttype: raise 'BadRequest', 'Not a %s field: %s' % (ttype, id)
            d={'id': id, 'type': propType}
            for f,t in fields.items():
                d[f] = field_value(id, f, t)
            return d
        def join_dicts(dict1, dict2):
            r={}
            for k,v in dict1.items() + dict2.items():
                r[k] = v
            return r
        pattern_tab = {}
        text_options={'case-sensitive': 'boolean',
                      'exact': 'boolean',
                      'all': 'boolean'}
        for f, t in (('pattern', 'string'), ('pattern', 'text'),
                     ('items', 'tokens'), ('items', 'lines'), ('items', 'meme')):
            pattern_tab[t] = join_dicts({f: t}, text_options)
        for t in ('int', 'long', 'float'):
            pattern_tab[t] = {'value': t, 'operator': 'string'}
        pattern_tab['boolean'] = {'on': 'boolean'}

        pattern_table = {'date': get_date_pattern}
        for propType,pat in pattern_tab.items():
            def tmp(id, propType=propType, pat=pat, get_some_pattern=get_some_pattern):
                return get_some_pattern(id, propType, pat)
            pattern_table[propType] = tmp

        def get_pattern(id, self=self, pattern_table=pattern_table):
            propType=self.propertyType(id)
            f=pattern_table.get(propType, None)
            if f: return f(id)
            else: raise 'BadRequest', 'Unknown pattern type: %s' % propType
            
        patterns=[]
        for id in self.propertyIds():
            if field_value(id, 'active'):
                patterns.append(get_pattern(id))
        aliases=self.getAllAliases(listOfLists=1)
        for ePat in patterns:
            if ePat['type']=='lines':
                kws=REQUEST.get('%s' % ('aliases_'+ePat['id']))
                if type(kws)!=type([]):
                    kws=[kws]
                kws=kws+ePat['items']
                self.__dbg(500, "kws: %s " % (kws))
                if REQUEST.get("%s/%s" % (ePat['id'],'aliasLookup')) is not None:
                    newKws=self.findAliases(aliases, kws)
                    ePat['items']=newKws
                else:
                    ePat['items']=self.removeDuplicates(kws)
              
        def read_bool(str):
            if str in ('0', '', 'no', None): return 0
            else: return 1
        allshares=read_bool(allshares)
        all=read_bool(all)
        start=None
        if allshares:
            share=None
            ob=self
            while 1:
                if getattr(ob, 'meta_type', None) == 'CARS Share':
                    share=ob
                if hasattr(ob, 'aq_parent'):
                    ob=ob.aq_parent
                else:
                    if not share:
                        raise 'BadRequest', 'Unable to find parent share.'
                    start=share
                    break
        else:
            start=self
        self.__dbg(100, "pattern: %s" %(patterns))
        found=findProperty([start], patterns, all=all)
        return self.findPropertyResult(self, result=process_result(found),
                                       URL=self.absolute_url())

    def addCARStoreFolder(self, id, title='', maxid=0, REQUEST=None):
        """Add a new store folder"""
        Log.carslog(self, 'addCARStoreFolder(%s)' % repr(id))
        if maxid:
            try: maxid = int(maxid)
            except: raise 'BadRequest', 'Invalid max. id: %s, integer expected.' % maxid
        ob=CARStoreFolder(id, title=title, maxid=maxid)
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.manage_main(self, REQUEST, update_menu=1)

    def removeObjectsFrom(self, container):
        self.__dbg(10, "removing from: %s" % container.id)
        cp=getSecurityManager().checkPermission
        container.refObjsCnt_to=0
        container.refObjsCnt_from=0
        container.cleanObjRefs()
        for ob in container.objectValues()[:]:
          if cp('Access contents information', ob):
            if ob.meta_type=="CARS Folder" or ob.meta_type=="CARS Share":
                ob.removeObjectsFrom(ob)
            else:
                self.__dbg(10, "deleting %s" % ob.id)
                container._delObject(ob.id)

    def manage_afterAdd(self, item, container):
        if self.isCloned():
            self.removeObjectsFrom(self)
        return CARSFolder.inheritedAttribute("manage_afterAdd") (self, item, container)

    def updateKw(self, id, reset=0):
        # for all objects in this folder, rebuild the keyword list for a
        # particular property id
        cp=getSecurityManager().checkPermission
        self.resetKwList(id)
        for ob in self.objectValues():
            self.__dbg(2000, "prop %s" % (ob.id))
            if cp('Access contents information', ob) and \
               ob.meta_type not in ('CARS Share', 'CARS Folder'):
                for prop in ob.fullPropertyMap():
                    if prop['id']==id:
                        self.__dbg(2000, "id: %s" % id) 
                        self.registerKw(id, ob.getProperty(id))
                        if reset:
                            if ob.getProperty(id)!=self.propertyDefault(id):
                                ob.setProperty(id, value=self.getKwList(id))
                                self.__dbg(20, "setting %s" % (ob.getProperty(id)))
        self.__dbg(20, "kwList: %s" % (self.getKwList(id)))

    def updateAllKws(self):
        # update all keywords in the current container
        self.__dbg(300, "updating all kws, %s" % (self.id))
        for prop in self.fullPropertyMap():
            self.modifyKw(prop['id'], prop['type'])

    def modifyKw(self, id, propType, reset=0):
        if propType=='lines':
            self.updateKw(id, reset=reset)

    def getCompleteProp(self, findType):
        """Return a complete list of values for a property from this object to the root."""
        r=[]
        ob=self
        while 1:
            for prop in ob.propertyMap():
                id=prop['id']
                propType=ob.propertyType(id)
                if propType==findType:
                    default=ob.getProperty(id)
                    r=r+filter(lambda x, r=r: not x in r, default) 
            if hasattr(ob, 'aq_parent') and hasattr(ob.aq_parent, 'subObjects'):
                ob=ob.aq_parent
            else:
                return r

    def getInheritedAliases(self):
        """Return a list of inherited Aliases"""
        r=[]
        ob=self
        while 1:
            if hasattr(ob, 'aq_parent') and hasattr(ob.aq_parent, 'subObjects') and \
               ob.inherited():
                ob=ob.aq_parent
                r=r+ob.rtnAliasList()
            else:
                break
        if r:
            return r
        return

    def getAliasList(self, REQUEST=None):
        r=[]
        l=self.rtnAliasList()
        self.__dbg(20, "aliasList: %s" % (l))
        for x in range(len(l)):
            r.append({'id': x, 'type': 'alias',
                      'label': 'Alias',
                      'value': l[x],
                      'default': '',
                      'seq': 0})
        return r
        
    def getAllAliases(self, listOfLists=0, REQUEST=None):
        """Return a list of all Aliases"""
        self.__dbg(10, "getAllAliases")
        r=[]
        al=self.rtnAliasList()
        il=self.getInheritedAliases()
        if il:
            al=al+il
        self.__dbg(5, "al list: %s" % (al))
        map(lambda l: l.sort(), al)
        if listOfLists:
            ## we need a list of lists
            ## first reverse the order so that the topmost ancestor
            ## appears from [0] decending to [n]
            ## needed to allow decendant to change the meaning of an alias
            al.reverse()
            return al
        nl=[]
        ## remember al is a list of lists and we need each element in the list
        ## placed into one list
        for l in al:
            for m in l:
                nl.append(m)
        self.__dbg(200, "found all Kws(): %s" % (self.findAllKws()))
        nl=nl+self.findAllKws()
        nl=self.removeDuplicates(nl)
        nl.sort()
        if nl:
            return nl
        return []

    def findAllKws(self):
        """Return a complete list of kws from this object to the root."""
        r=[]
        ob=self
        while 1:
            r=r+ob.rtnGeneralKws()
            if hasattr(ob, 'aq_parent') and hasattr(ob.aq_parent, 'subObjects'):
                ob=ob.aq_parent
            else:
                return r
            
    def getAllKws(self, ref, r=[]):
        """Return a list of kws that are found in objects here and below"""
        r=self.removeDuplicates(self.findKwsBelow(ref))
        return r

    def findKwsBelow(self, ref, r=[]):
        """Return a list of kws that are found in objects here and below"""
        cp=getSecurityManager().checkPermission
        for ob in self.objectValues():
            if cp('Access contents information', ob) and \
               ob.meta_type in ('CARS Share', 'CARS Folder'):
                r=r+ob.findKwsBelow(ref, r=r)
        self.__dbg(200, "returning %s" % (r))
        return r+self.getKwList(ref)
    
    def getMemeList(self, id):
        ml=self.findParentMemes(id)
        self.__dbg(500, "All Memes for id %s: %s " % (id, ml))
        r=[]
        for meme in ml:
            r.append({'id': id, 'idx': meme['idx'],
                      'memeText': meme['text']})
        return r

    def findParentMemes(self, id, dict=0, findMode=0):
        """Return a complete list of kws from this object to the root."""
        r=[]
        ob=self
        while 1:
            meme=ob.rtnMeme(id, dict=dict)
            if meme:
                # return this first ancestor with meme list
                if findMode:
                    return ob.findMemeMode(id)
                return meme
            if hasattr(ob, 'aq_parent') and hasattr(ob.aq_parent, 'subObjects'):
                ob=ob.aq_parent
            else:
                return r

    def memeMode(self, id):
        '''Future function?  Enabling lists to select either multiple or single elements
        '''
        ## All memes should be single selection at the moment
        return None
        r=self.findParentMemes(id, findMode=1)
        if r:
            return r
        return None

    def findMemeMode(self, id):
        mode=self.getMultiMode(id)
        ## if the mode is '' empty then latte needs 
        if mode=='':
            mode=None
        else:
            mode=1
        self.__dbg(500, "memeMode: %s " % mode)
        return mode

    def isPropertyContainerDefault(self, id, value):
        """Return true if 'value' is the same as the container Default"""
        for prop in self.containerPropertyMap():
            if prop['id']==id:
                try:
                    if prop['type']=='meme':
                        self.__dbg(200, "meme Value %s" % (value))
                        if value==None:
                            return 1
                    else:
                        self.__dbg(5, "def %s, value %s" % (prop['default'], value))
                        if prop['default']==value:
                            self.__dbg(100, "found def: %s, for %s" % (value, id))
                            return 1
                except:
                    pass
        if hasattr(self, 'aq_parent') and hasattr(self.aq_parent, 'subObjects'):
            self.__dbg(100, "calling parent %s, %s" % (self.aq_parent.id, value))
            return self.aq_parent.isPropertyContainerDefault(id, value)
        return 0

    def CARSobjectValues(self):
        """Return a list of folders from current down"""
        cp=getSecurityManager().checkPermission
        r=[]
        for ob in self.objectValues():
            if cp('View', ob):
                if ob.meta_type=="CARS Folder" or ob.meta_type=="CARS Share":
                    r.append(ob)
        return r

    def manage_beforeDelete(self, item, container):
        self.__oldPath=self.destructExternFSpath(self.absolute_url(1))
        CARSFolder.inheritedAttribute("manage_beforeDelete") (self, item, container)

    def manage_afterAdd(self, item, container):
        newPath=self.absolute_url(1)
        self.reconstructExternFSpath(self.__oldPath, newPath)
        self.__oldPath=''
        CARSFolder.inheritedAttribute("manage_afterAdd") (self, item, container)

    def removeAllSubObjects(self):
        """Remove all objects in this container and below.  Only the structure
        should remain
        """
        cp=getSecurityManager().checkPermission
        for ob in self.objectValues():
            if cp('View', ob):
                if ob.meta_type=="CARS Folder" or ob.meta_type=="CARS Share":
                    ob.removeAllSubObjects()
                else:
                    self.__dbg(500, "deleting: %s" % (ob.id))
                    self._delObject(ob.id)



manage_addCARStoreFolderForm = HTMLFile('carstorefolderAdd', globals())

def manage_addCARStoreFolder(self, id, title='', maxid=0, REQUEST=None):
    """Add an CARS Store Folder to a folder."""
    Log.carslog(self, 'addCARStoreFolder(%s)' % repr(id), REQUEST=REQUEST)
    if maxid:
        try: maxid = int(maxid)
        except: raise 'BadRequest', 'Invalid max. id: %s, integer expected.' % maxid
    ob=CARStoreFolder(id, title=title, maxid=maxid)
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class CARStoreFolder(CARSFolder):
    meta_type = 'CARS Store Folder'

    lastid = -1
    maxid = 0

    def __init__(self, id, title='', maxid=0):
        self.id = str(id)
        self.lastid = 1
        self.maxid = maxid
        self.title = title

    def make_id(self, REQUEST=None):
        """Generate a new unique object id"""
        newid=self.genId()
        while(hasattr(self, newid)):
            newid=self.genId()
        return newid

