__Object__ = """CARSObject product module."""
__version__ = '0.1'

import string
from StringIO import StringIO
from Globals import HTMLFile,MessageDialog,Persistent
import OFS.Folder
import OFS.Image
import Acquisition
import AccessControl
from OFS.content_types import guess_content_type
from Products.CARS.PropertyManager import PropertyClient, PropertyStore
from Products.CARS import Log
from Products.CARS.CARSFolder import CARSFolder
from Products.CARS.objBase import objBase
from Products.CARS.externFS import externFS
from AccessControl import getSecurityManager
from time import strftime, localtime, time

timeformat='%Y-%m-%d %H:%M:%S'

def timestr():
    global timeformat
    return strftime(timeformat, localtime(time()))

class CARSObject(
    PropertyClient.PropertyClient,
    PropertyStore.PropertyStore,
    objBase.objBase,
    OFS.ObjectManager.ObjectManager,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    OFS.SimpleItem.Item
    ): 
    
    __dbgLevelDiagnostic = 0
    __className='CARSObject'
    __oldPath=''
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    __ac_permissions__=(
        ('View', ('index_html', 'manage_workspace', 'manage_main', 'print_html',
                  'manage_printWithLabelsForm', 'manage_printWithoutLabelsForm',
                  'manage_referencesForm')),
        ('Logout', ('manage_logout',)),
        ('Add Pages', ('manage_addPage', 'manage_edit',
                       'manage_updatePage', 'manage_delPage',
                       'manage_delPages', 'manage_attachDoc',
                       'manage_removeAttachedDoc')),
        ('Add CARS Files', ('manage_uploadFile', 'manage_edit',
                           'manage_delFileId', 'manage_delFiles')),
        ('Authenticate Signature', ('manage_authenticateSignatureForm',
                                    'manage_authenticateSignature')),
        ('Manage properties', ('manage_propertiesForm', 'manage_editProperties')))

    manage_options = (
	{'label': 'View',       'action': 'index_html', 'image': 'view'}, 
        {'label': 'References', 'action': 'manage_referencesForm',
         'image': 'reference', 'frame': '_blank'},
	{'label': 'Attach',       'action': 'manage_edit', 'image': 'edit'},
        {'label': 'Properties', 'action': 'manage_propertiesForm', 
         'image': 'properties'},
	{'label': 'Search', 'action': 'manage_findPropertyForm',
	 'image': 'search'},
        {'label': 'Print',  'action': 'print_html', 'image': 'print'},
        {'label': 'Sign', 'action': 'manage_authenticateSignatureForm', 'image': 'sign'},
        {'label': 'Log out', 'action': 'manage_logout', 'image': 'logout'})

    title=''

    def __init__(self, id, title='', revisioNum=1):
        CARSObject.inheritedAttribute("__init__") (self)
        self.id = id
        self.title = title
        self.revisioNum=revisioNum
        self.__dbg(1, "Initialised object")

    index_html = HTMLFile('index', globals()) # View Interface
    print_html = HTMLFile('print', globals()) # output data for printing
    manage_referencesForm=HTMLFile('references', globals())
    manage_printWithLabelsForm = HTMLFile('printWlabels', globals())
    manage_printWithoutLabelsForm = HTMLFile('printWOlabels', globals())
    manage_authenticateSignatureForm = HTMLFile('sign', globals()) # Authenticate user signature
    manage_main = HTMLFile('index', globals()) # View Interface
##    manage_edit = HTMLFile('edit', globals()) # Management Interface
    manage_propertiesForm = HTMLFile('../PropertyManager/properties', globals())
    manage_logout=CARSFolder.CARSFolder.manage_logout

    def appendPermissions(self, ac_perm, extra_perm):
        ac_perm=list(ac_perm)
        extra_perm=list(extra_perm)
        for e in extra_perm:
            ac_perm.append(e)
        return tuple(ac_perm)

    def manage_authenticateSignature(self, comment='', passwd='', REQUEST=None):
        """Authenticate a users signature allowing them to stamp a
        document with their username.
        """
        if not hasattr(self, '_signatures'):
            self._signatures=[]
        thisUser=getSecurityManager().getUser()
        self._signatures.append("%s [%s]: %s" % (timestr(), thisUser, comment))
        self._p_changed=1
        Log.carslog(self, 'Object(%s) signed by %s' % (repr(self.id), repr(thisUser)),
                    REQUEST=REQUEST)
        if not self.isImmutable():
            self.setImmutable()
            Log.carslog(self, 'Object(%s) made immutable.' % repr(self.id), REQUEST=REQUEST) 
            self.informContacts("Immutable", "Object %s is now immutable."%self.id)
        if REQUEST:
            return self.manage_main(self, REQUEST)

    def getSignatures(self):
        """Return signatures"""
        if hasattr(self, '_signatures'):
            return self._signatures
        return None

    def manage_afterAdd(self, item, container):
        if self.isCloned():
            container._delObject(self.id)
        else:
            container.updateAllKws()
        return CARSObject.inheritedAttribute("manage_afterAdd") (self, item, container)


class CARSeXternFS(CARSObject,
                   externFS.externFS
                   ):
    __dbgLevelDiagnostic = 0
    __className='CARSeXternFS'
    __oldPath=''

    def __init__(self, id, title, revisioNum): 
        CARSeXternFS.inheritedAttribute("__init__") (self, id, title, revisioNum)
        self.extFSList=[]
        self.undeleteFSList=[]
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    def createXternFS(self, file, filename, title='', \
                      content_type='', subPath=''):
##         filename=string.split(string.replace(filename, " ", "_"), '\\')[-1]
##         file.filename=string.split(string.replace(file.filename, " ", "_"), '\\')[-1]
        return externFS.externFS(file, filename, title=title, \
                                 content_type=content_type, \
                                 subPath=subPath, \
                                 urlPath=self.absolute_url(1))

    def appendeXternFS(self, file, filename, title='', \
                       content_type='', subPath=''):
        self.extFSList.append(self.createXternFS(file, filename, title=title,\
                                                 content_type=content_type, \
                                                 subPath=subPath))
        self._p_changed=1

    def removeXternFS(self, filename, subPath=''):
        url=self.absolute_url(1)
        for eFS in self.extFSList[:]:
            self.__dbg(1, "Found %s" %(eFS.getFilename()))
            if filename == eFS.filename and subPath == eFS.subPath:
                self.__dbg(1, "Attempting to delete: %s" % (eFS.getFilename()))
                eFS.delete(url)
                self.extFSList.remove(eFS)
        self._p_changed=1

    def deleteSubPathFS(self, subPath):
        '''Remove files stored in subPath'''
        url=self.absolute_url(1)
        for eFS in self.extFSList[:]:
            if subPath==eFS.subPath:
                eFS.delete(url)
                self.extFSList.remove(eFS)
        self._p_changed=1

    def deleteXternFS(self, FSList):
        url=self.absolute_url(1)
        for eFS in FSList[:]:
            self.__dbg(1, "Attempting to delete: %s" % (eFS.getFilename()))
            eFS.delete(url)
        return []

    def undeleteXternFS(self, FSList):
        url=self.absolute_url(1)
        for eFS in FSList:
            self.__dbg(1, "Attempting to undelete: %s" % (eFS.getFilename()))
            eFS.undelete(url)
        return FSList[:]

    def duplicateFS(self, FSList):
        nFSList=[]
        for eFS in FSList:
            try:
                file=open(eFS.getFullPath(self.absolute_url(1)), 'rb')
            except IOError:
                raise IOError, "Cannot open %s" % (self.id)

            nFSList.append(self.createXternFS(file, eFS.filename, \
                                              content_type=eFS.content_type, \
                                              subPath=eFS.subPath,\
                                              urlPath=self.absolute_url(1)))
            self.__dbg(1, "duplicated file %s" %(nFSList[-1].getFullPath()))
        return nFSList

    def geteXternFS(self, filename, subPath=''):
        file=StringIO()
        eFS=self.geteXternFShandle(filename, subPath)
        if eFS:
            self.copyFile(eFS, file, urlPath=self.absolute_url(1))
        return file.getvalue()

    def geteXternFShandle(self, filename, subPath=''):
        for eFS in self.extFSList[:]:
            if filename == eFS.filename and subPath == eFS.subPath:
                return eFS
        return

    def geteXternFShandleList(self, subPath=''):
        r=[]
        for eFS in self.extFSList:
            if subPath == eFS.subPath:
                r.append(eFS)
        return r

    def geteXternFilenameList(self, subPath=''):
        r=[]
        for eFS in self.extFSList:
            if subPath == eFS.subPath:
                r.append(eFS.filename)
        return r

    def existFSList(self):
        if self.extFSList:
            return 1
        return 0

    def REQUESTeXternFS(self, filename, subPath, REQUEST=None):
        """GetFileLink"""
        eFS=self.geteXternFS(filename, subPath=subPath)
        eFSh=self.geteXternFShandle(filename, subPath=subPath)
        Log.carslog(self, 'showFile(%s)' % filename, REQUEST=REQUEST)
        if REQUEST:
            REQUEST['RESPONSE'].setBody(eFS)
            REQUEST['RESPONSE'].setHeader('Content-Type', eFSh.content_type)
#	    REQUEST['RESPONSE']=self.showFileForm(self, ob.data=ob.data)
#        self.__dbg(200, "returning REQexternFS: %s" % (REQUEST))
        return eFS

    def getFilePath(self, filename, subPath=''):
        eFS=self.geteXternFShandle(filename, subPath)
        if eFS:
            return eFS.getFullPath(self.absolute_url(1))
        return

##     def _getFileSize(self, filename, subPath=''):
##         eFS=self.geteXternFShandle(filename, subPath)
##         if eFS:
##             return eFS.size
##         return

    def geteXternFSDetails(self, subPath):
        """Return filename and filesize details"""
        r=[]
        FShList=self.geteXternFShandleList(subPath)
        self.__dbg(200, "FShList: %s, subPath: %s, self %s" % (FShList, subPath, self))
        if len(FShList)<1:
            fn=''
            size='None'
            return [{'filename': fn, 'size': size}]
        app=self.getPhysicalRoot()
        for eFS in FShList:
            icon=eFS.getIcon(app)
            r.append({'filename': eFS.filename, 'size': eFS.size, 'icon':icon})
        self.__dbg(200, "returning : %s" % (r))
        return r

    def manage_beforeDelete(self, item, container):
        self.__dbg( 500, "CARSeXtern: b4Delete: %s, %s" % (item.id, item.absolute_url(1)))
        self.undeleteFSList=self.extFSList[:]
        self.extFSList=self.deleteXternFS(self.extFSList)
        self.__oldPath=self.destructExternFSpath(self.absolute_url(1))
        self._p_changed=1
        return CARSeXternFS.inheritedAttribute ("manage_beforeDelete") (self, item, container)

    def manage_afterAdd(self, item, container):
        self.__dbg( 500, "CARSeXtern: aftRAdd: %s, %s" % (item.id, item.absolute_url(1)))
        newPath=self.absolute_url(1)
        self.reconstructExternFSpath(self.__oldPath, newPath)
        self.__oldPath=''
        self.extFSList=self.undeleteXternFS(self.undeleteFSList)
        self.undeleteFSList=[]
        self._p_changed=1
        return CARSeXternFS.inheritedAttribute ("manage_afterAdd") (self, item, container)

#    def manage_afterClone(self, item):
#        self.__dbg( 5, "CARSeXtern: aftRClone: %s" %(self.id))
#        self.extFSList=self.duplicateFS(self.extFSList)
#        self.undeleteFSList=self.extFSList[:]
#        self.extFSList=self.deleteXternFS(self.extFSList)
#        self._p_changed=1
#        return CARSeXternFS.inheritedAttribute ("manage_afterClone") (self, item)





