__doc__ = """CARShare product module."""
__version__ = '0.1'

import nis,traceback,string
from Globals import HTMLFile,MessageDialog,Persistent
from Products.CARS.CARSFolder import CARSFolder
from Products.CARS.NisLogin import NisLogin
from Products.LoginManager.LoginManager import manage_addLoginManager
from Globals import HTMLFile
from Products.CARS import Log
from Products.CARS.objBase import objBase

read_access=['View', 'Find Properties', 'Access contents information', 'Logout']
property_access=['Manage properties']+read_access
security_access=['Edit Access Rights']+read_access
signature_access=['Authenticate Signature']+read_access
publish_access=['Add CARS File Group', 'Publish Doc', 'Add CARS Document',
                'Add CARS Record', 'Add CARS Files', 'Add Pages', 'Forward Doc']+read_access
upload_access=['Add CARS Store Folders']+publish_access
edit_access=['Delete objects', 'Copy/Paste', 'Rename objects', 'Add Pages']+publish_access
build_access=['Add CARS Shares', 'Add CARS Folders', 'Add NIS User Source',
              'Manage property map']+edit_access

def rmDuplicates(perms):
    def strip(x, l=[]):
        if x not in l:
            l.append(x)
            return x
        return
    return filter(strip, perms)

manage_addCARShareForm = HTMLFile('carshareAdd', globals())

def manage_addCARShare(self, id, title='', revisioNum=1, REQUEST=None):
    """Add an CARS Share to a folder."""
    Log.carslog(self, 'addCARShare(%s)' % repr(id), REQUEST=REQUEST)
    ob=CARShare()
    ob.id=id
    ob.title=title
    ob.revisioNum=revisioNum
    self._setObject(id, ob)
    ob=self._getOb(id)
    manage_addLoginManager(ob, usource='NIS User Source', loginf=0, logoutf=0, forbidp=0)
    acl=ob._getOb('acl_users')
    acl.manage_acquiredPermissions(permissions=[])
    for id in ob.getPermissionsGroupIds():
        ob._addRole(id)
        perms=ob.getPermissions(id)
        ob.manage_role(id, permissions=perms)
        acl.manage_role(id, permissions=[])
    ob.manage_role('Owner', permissions=read_access)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

class CARShare(CARSFolder.CARSFolder):

    global read_access, property_access, publish_access, \
           edit_access, build_access, upload_access, security_access, signature_access,\
           read_access_permissions, write_access_permissions, upload_access_permissions,\
           rmDuplicates
    
    meta_type = 'CARS Share'

    __dbgLevelDiagnostic = 0
    __className='CARShare'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    fullname_tab=None
    extraPermissions=[('Edit Access Rights',
                          ('manage_addUserAccess', 'manage_editAccess', 'manage_removeAccess'))]
   
    ac_permissions=list(CARSFolder.CARSFolder.__ac_permissions__)
    for e in extraPermissions:
        ac_permissions.append(e)
    __ac_permissions__=tuple(ac_permissions)

    manage_options = CARSFolder.CARSFolder.manage_options + \
    ({'label': 'Security', 'action': 'manage_editAccess', 'image': 'security'},)

    manage_editAccess=HTMLFile('editAccess', globals())

    __groupPermissions=None
    __usersList=None
    __groupsList=None

    def _initPropMap(self):
        self._property_map=({'id': 'title', 'label': 'Title',
                             'type': 'string', 'mode': 'w', 'seq': 1},)

    def __init__(self):
        CARShare.inheritedAttribute("__init__") (self)
        self._initPropMap()
        self.fullname_tab={}
        self.__groupPermissions={}
        self.__usersList={}
        self.__groupsList={}
        self.initPermissions()
        self.__dbg(1, "Initialised Share")

    def initPermissions(self):
        self.setPermissions('Read Access', rmDuplicates(read_access))
        self.setPermissions('Property Access', rmDuplicates(property_access))
        self.setPermissions('Publish Access', rmDuplicates(publish_access))
        self.setPermissions('Edit Access', rmDuplicates(edit_access))
        self.setPermissions('Build Access',rmDuplicates( build_access))
        self.setPermissions('Upload Access', rmDuplicates(upload_access))
        self.setPermissions('Security Access', rmDuplicates(security_access))
        self.setPermissions('Signature Access', rmDuplicates(signature_access))

    def propertyMap(self):
        if self._inherited:
            return CARShare.inheritedAttribute("propertyMap") (self)
        return self._property_map

    def appendPropMap(self, m=None):
        """Add old properties found in the parent map
        """
        def diffMap(ancestorMap, siblingMap):
            """Return a list of properties to add to current share
            """
            def findProp(aMap, xId):
                for a in aMap:
                    if a['id']==xId:
                        return a
                return None
            for e in siblingMap:
                try:
                    ancestorMap.remove(findProp(ancestorMap, e['id']))
                except:
                    ## no property in ancestor map
                    pass
            return ancestorMap
        self._appendPropMap(diffMap(list(m), self.propertyMap()))

    def add_user_access(self, users, access):       
        try:
            for user in string.split(users, " "):
                if not string.lower(user) == "anonymous":
                    nis.match(user, 'passwd.byname')
                rtnErr=self.addUser(user, access)
                if rtnErr:
                    raise 'BadRequest', rtnErr               
        except nis.error:
            raise 'BadRequest', 'No such user: %s.' % user
        return MessageDialog(
            title = 'Success!',
            message = '%s access for %s has been successfully added.' \
            % (access, users),
            action = 'manage_main')

    def remove_user_access(self, user, access=None):
        try:
            self.removeUser(user, accessList=access)
        except ValueError:
            if not access:
                access='<unknown>'
            raise 'BadRequest', 'User %s doesn\'t have %s access.' % (user, access)
        
    def manage_addAccess(self, users, access):
        return self.add_user_access(users, access)

    def manage_delAccess(self, user, access=''):
        return self.remove_user_access(user, access=access)

    def _hasAccess(self, user):
        return self.groupsList(user)
        
    def _hasAnyAccess(self, user):
        """Return a list of access rights for a user, from the current
        container all the way to the root object
        """
        r=[]
        ob=self
        while 1:
            access=ob._hasAccess(user)
            if access:
                ## filter duplicate access rights
                r=r+filter(lambda x, r=r: not x in r, access) 
            if hasattr(ob, 'aq_parent') and hasattr(ob.aq_parent, 'subObjects'):
                ## use parent
                ob=ob.aq_parent
            else:
                ## return a list of access rights
                return r
    
    def manage_removeAccess(self, REQUEST=None):
        """Remove access for the selected users"""
        s=''
        for g in self.getGroupList():
            users=REQUEST.get(g,'')
            if not type(users)==type([]):
                users=[users]
            if users:
                for u in users:
                    self.manage_delAccess(u, g)
                s=s+g+'='+repr(users)+'; '

        Log.carslog(self,
                    'removeAccess(%s)' \
                    % (repr(s)),
                    REQUEST=REQUEST)
        if REQUEST:
            REQUEST['RESPONSE'].redirect('manage_editAccess')

    def manage_addUserAccess(self, users, groups=[], REQUEST=None):
        """Add access for a user"""
        if not groups:
            raise 'BadRequest', 'Expected at least one out of read, write or upload access.'
        s=''
        for g in groups:
            self.manage_addAccess(users, g)
            s=s+g+'; '

        Log.carslog(self,
                    'addUserAccess(%s, %s)' \
                    % (repr(users), repr(s)),
                    REQUEST=REQUEST)
        if REQUEST:
            REQUEST['RESPONSE'].redirect('manage_editAccess')

    def getGroupList(self):
        return self.getPermissionsGroupIds()

    def getUserAccessList(self, access=[]):
        if access:
            return self.usersList(access)
        return self.usersList('Read Access')
    
    def getFullName(self, user):
        if user == "anonymous":
            return 'everyone'
        return string.split(string.split(nis.match(user, 'passwd.byname'), ':')[4], ',')[0]
    
    def setPermissions(self, group, permissions):
        """Place group in dictionary with permissions.
           If the group already exists then update permissions.
        """
        self.__groupPermissions[group]=permissions

    def getPermissions(self, group):
        """Return the list of permissions for a requested group
        """
        if self.__groupPermissions.has_key(group):
            return self.__groupPermissions[group]
        return None

    def getPermissionsGroupIds(self):
        """Return a list of groups available in the permissions list
        """
        return self.__groupPermissions.keys()

    def addUser(self, user, access):
        """Add a user to a group
        """
        if self.__usersList.has_key(user):
            if not filter(lambda x, access=access: x == access, self.__usersList[user]):
                self.__usersList[user].append(access)
            else:
                return 'User (%s) already has %s' % (user, access)
        else:
            self.__usersList[user]=[access]

        if self.__groupsList.has_key(access):
            if not filter(lambda x, user=user: x == user, self.__groupsList[access]):
                self.__groupsList[access].append(user)
            else:
                return 'Group (%s) already has user %s' % (access, user)
        else:
            self.__groupsList[access]=[user]
        self._p_changed=1
        return None

    def removeUser(self, user, accessList=None):
        """Remove a user from a group.  If none supplied then remove from all groups.
        """
        def removeItem(item, list):
            return filter(lambda x, item=item: item != x, list)
            
        if self.__usersList.has_key(user):
            if not accessList:
                accessList = self.__usersList[user]
            ## check if accessList is a proper list, else create one
            if not type(accessList)==type([]):
                accessList=[accessList]
            for access in accessList:
                self.__usersList[user].remove(access)
                self.__groupsList[access].remove(user)
            self._p_changed=1        

    def usersList(self, access):
        """Return a list of users for a given group access
        """
        if self.__groupsList.has_key(access):
            return self.__groupsList[access]
        return None

    def groupsList(self, user):
        """Return a list of access groups a given user has rights to
        """
        if self.__usersList.has_key(user):
            return self.__usersList[user]
        return None
        
