import FCNTL
from fcntl import flock
from time import strftime, localtime, time
import sys, syslog, string
from syslog import LOG_EMERG, LOG_ALERT, LOG_CRIT, LOG_ERR, LOG_WARNING, LOG_NOTICE, LOG_INFO, LOG_DEBUG
from os import stat

logname='/tmp/CARS.log'
logfile=None
loglevel=0
timeformat='%Y-%m-%dT%H:%M:%S'
buffer=[]
syslog_open=0
inode=None

log_levels=[LOG_EMERG, LOG_ALERT, LOG_CRIT, LOG_ERR, LOG_WARNING, LOG_NOTICE, LOG_INFO, LOG_DEBUG]

def timestr():
    global timeformat
    return strftime(timeformat, localtime(time()))

def openlog():
    global logfile, logname, inode
    logfile=open(logname, 'a+')
    inode=stat(logname)[1]
    return logfile

def update_logfile():
    global logfile, logname, inode
    try: r = (inode != stat(logname)[1])
    except: r = 1
    if r:
        logfile.close()
        return openlog()
    else:
        return logfile

def flush_buffer():
    global buffer
    for l in buffer:
        logmsg(LOG_ALERT, l)
    buffer = []

def path(ob):
    try:
        return ob.absolute_url(1)
    except:
        pass
    result=[]
    def getid(ob):
        return str(getattr(ob, 'id', '???'))
    while hasattr(ob, 'aq_parent') and hasattr(ob.aq_parent, 'aq_parent'):
        result.insert(0, ob.aq_base)
        ob=ob.aq_parent
    return string.join(map(getid, result), '.')

def format_msg(ob, msg, user):
    def sanify(stri):
        return string.replace(str(stri), ' ', '\\ ')
    return '[%s] %s %s' % (sanify(user or ''), sanify(path(ob)), msg)

def logmsg_file(level, msg):
    global logfile, timeformat, logname, log_levels
    if logfile: logfile=update_logfile()
    else: logfile=openlog()
    flock(logfile.fileno(), FCNTL.LOCK_SH)
    try:
        logfile.write('%s %d %s\n' % (timestr(), log_levels.index(level), msg))
        logfile.flush()
    finally: flock(logfile.fileno(), FCNTL.LOCK_UN)

def level2syslog(level):
    if level in log_levels:
        return level
    else: raise ValueError, 'Invalid log level'

def logmsg_syslog(level, msg):
    global syslog_open
    if not syslog_open:
        syslog.openlog('CARS')
    syslog.syslog(level2syslog(level), msg)

logmsg=logmsg_file

def set_logmethod(method):
    global logmsg
    if method == 'syslog':
        logmsg=logmsg_syslog
    elif method == 'file':
        logmsg=logmsg_file
    else:
        raise ValueError, 'Invalid logging method'

def carslog(ob, msg, level=LOG_INFO, user=None, REQUEST=None):
    u=user
    try:
        if (not u) and REQUEST: u=REQUEST._authUserPW()[0]
        flush_buffer()
        logmsg(level, format_msg(ob, msg, user=u))
    except:
        (type,val) = sys.exc_info()[:2]
        buffer.append(format_msg(ob, 'Logging failure at %s, %s: %s' \
                                 % (timestr(), type, val), user))
        buffer.append(format_msg(ob, msg, user=user))
        raise

def debug(dbgLevelDiagnostic, level, dbgLine, className):
    if dbgLevelDiagnostic and level>=dbgLevelDiagnostic:
        print "[%s] %s: %s" % (className, level, dbgLine)

