import ExtensionClass
import string, sys
from DateTime import DateTime
from ZPublisher.Converters import get_converter
from Products.CARS import Log

class PropertyBase(ExtensionClass.Base):

    __dbgLevelDiagnostic = 0
    __className='PropBase'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    def _wrapperCheck(self, object):
        # Raise an error if an object is wrapped.
        if hasattr(object, 'aq_base'):
            raise ValueError, 'Invalid property value: wrapped object'
        return

    def rmWrapper(self, object):
        if hasattr(object, 'aq_base'):
            return object.aq_base
        return object

    def getPropertyType(self, id):
        """Get the type of property 'id', returning None if no
           such property exists"""
        md = self.getPropertyDef(id)
        if md:
            return md.get('type', 'string')
        return None

    def isUsingDefault(self, id):
        """Is the object currently using the default"""
        return self.isPropertyContainerDefault(id, self.getProperty(id))

    def propertyType(self, id):
        return self.getPropertyType(id)

    def propertySeq(self, id):
        """Return a sequence no for the given property id
        """
        md = self.getPropertyDef(id)
        if md:
            return md.get('seq', 0)
        return None

    def getPropertyDef(self, id):
        for md in self.propertyMap():
            if md['id']==id:
                return md
        return None

    def _samePropertyDef(self, id, default, label, type):
        return (propertyType(id) == type and \
                propertyLabel(id) == label and \
                propertyDefault(id) == default)

    def propertyLabel(self, id):
        """Return a label for the given property id
        """
        md = self.getPropertyDef(id)
        if md:
            return md.get('label', str(id))
        return None

    def compareType(self, id, type1):
	return (self.propertyType(id) == type1)

    def typeDefault(self, type):
        if type in ('int', 'float', 'long', 'boolean'):
            return 0
        elif type == 'date':
            return DateTime(0)
        elif type in ('tokens', 'lines', 'inform', 'meme'):
            return ''
        elif type in ('string', 'text', 'email'):
            return ''
        elif type == 'objref':
            return 'None'
        else:
            return None

    def read_property(self, value, ofType, id='<unknown>'):
        if ofType=='inform':
            ofType='tokens'
        if ofType=='meme':
            ofType='list'
        if ofType=='objref':
            try:
                app=self.getPhysicalRoot()
                ## return the unwrapped object
                return app.unrestrictedTraverse(value).aq_base
            except:
                return None
        if (ofType=='date' or ofType in ('int', 'float', 'long', 'boolean')) and \
           value=='':
            return ''
            
        try: r=(get_converter(ofType, lambda x: x))(value)
        except:
            val = sys.exc_info()[1]
            raise 'ValueError', \
                  'Error parsing property value for %s (%s): %s.' % (id, ofType, val)
        return r

    def propertyDefault(self, id):
        """Return the default value for the given property id
        """
        md = self.getPropertyDef(id)
        if md:
             return md.get('default', self.typeDefault(self.propertyType(id)))
        return None

    def _formatValue(self, val, type):
        """Return a formatted property value"""
        if val is None: return ''
        if type in ('int', 'float', 'string', 'date', 'email'):
            return '%s' % val
        elif type == 'long':
            return ('%s' % val)[:-1]
        elif type == 'boolean':
            if val:
                return 'CHECKED'
            else:
                return ''
        elif type in ('tokens', 'inform'):
            return '%s' % string.join(val, ' ')
        elif type == 'text':
            return val
        elif type in ('lines', 'alias', 'meme'):
            if len(val)>0:
                return string.join(val, '\n')
            else:
                return ''
	elif type == 'objref':
	    return '%s' % val
        else:
            return 'Unknown property type %s' % type

    def formatProperty(self, id, type=None):
        if type is None:
            return self._formatValue(self.getProperty(id), self.propertyType(id))
        return self._formatValue(id, type)

    def formatPropertyDefault(self, id):
        return self._formatValue(self.propertyDefault(id),
                                 self.propertyType(id))

    def valid_property_map_id(self, id):
        if not id or id[:1]=='_' or (' ' in id):
            return 0
        return 1

    def hasProperty(self, id):
        """Return true if object has a property 'id'"""
        for p in self.propertyMap():
            if p['id'] == id:
                return 1
        return 0

    def propertyIds(self):
        """Return a list of property ids"""
        return map(lambda i: i['id'], self.propertyMap())
    
    def propertyValues(self):
        """Return a list of actual property objects"""
        return map(lambda id, self=self: self.getProperty(id),
                   self.propertyIds())

    def propertyItems(self):
        """Return a list of (id,property) tuples """
        return map(lambda id, self=self: (id,self.getProperty(id)),
                   self.propertyIds())

    def propdict(self):
        dict={}
        for p in self.propertyMap():
            dict[p['id']]=p
        return dict
