from PropertyBase import PropertyBase
from Globals import HTMLFile, MessageDialog
from ZPublisher.Converters import type_converters
import sys
import string
from Products.CARS import Log

class PropertyServer(PropertyBase):

    _property_map = ()
    _inherited=0
    __dbgLevelDiagnostic = 0
    __className='PropSrvr'

    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    def propertyMap(self):
        return self._property_map

    def fullPropertyMap(self):
        """x"""
        r=[]
        for id in self.propertyIds():
            r.append({'id': id, 'type': self.propertyType(id),
                      'label': self.propertyLabel(id),
                      'default': self.formatPropertyDefault(id),
                      'seq': self.propertySeq(id)})
        return r

    def _setMultiMode(self, id, mode):
        for md in self._property_map:
            if md['id']==id:
                md['multiMode']=mode
                return md

    def getMultiMode(self, id):
        p=self._localPropertyDef(id)
        if p: return p['multiMode']
        return None
    
    def _setPropertyDef(self, id, default, label=None, propType=None, seq=None):
        for md in self._property_map:
            if md['id'] == id:
                # Setting always to default value
                md['default'] = default
                if label: md['label'] = str(label)
                if propType: md['type'] = propType
                if seq != None: md['seq'] = float(seq)
                self._p_changed=1
                return md
        self.__dbg(200, "_setPropDef: %s" % (self._property_map))
        raise 'Bad Request', 'Set property definition: Invalid property id: %s' % id
            
    def _addPropertyDef(self, id, default, label, propType, seq):
        self._wrapperCheck(default)
        if (not self.valid_property_map_id(id)) or self._localPropertyDef(id):
            raise 'Bad Request', \
                  'Add property definition: Invalid or duplicate property id: %s' % id
        if not propType: raise 'BadRequest', 'Missing property type'
        d={'id': id, 'type': propType}
        d['default'] = default
        if label: d['label'] = label
        if seq: d['seq'] = float(seq)
        self._property_map = self._property_map + (d,)
        self._p_changed=1

    def _appendPropMap(self, map=None):
        if map:
            for m in map:
                self._addPropertyDef(m['id'], m.get('default'), m.get('label'),
                                     m['type'], m.get('seq'))

    def _delPropertyDef(self, id):
        self._property_map = tuple(filter(lambda x,id=id: x['id'] != id, self._property_map))

    def _localPropertyDef(self, id):
        for md in self._property_map:
            if md['id'] == id:
                return md
        return None
    
    def _deleteablePropertyDef(self, id):
        p=self._localPropertyDef(id)
        if p: return 'd' in p.get('mode', 'wd')
        return None

    def isDeleteablePropertyDef(self, id, REQUEST=None):
        return self._deleteablePropertyDef(id)

    def inherited(self):
        return self._inherited

    def manage_editPropertyMap(self, REQUEST):
        """Edit property map via the web.
        """
        for prop in self.propertyMap():
            if 'w' in prop.get('mode', 'wd'):
                ## get default value of property from request
                id=prop['id']
                propType=self.propertyType(id)
                defaultReq=REQUEST.get(id, '')
                default=self.read_property(defaultReq, propType, id=id)
                if propType == 'objref':
                    ## then this is a reference which must be handled differently
                    ## as it contains instance references which change constantly
                    listReq=REQUEST.get('list_'+str(id), '')
                    listSelection=self.read_property(listReq, propType, id=id)
                    self.__dbg(1, "\nlistSel: %s\n defReq: %s\n default: %s" % \
                           (listSelection, default, self.propertyDefault(id)))
                    propDef=self.propertyDefault(id)
                    propDef=self.rmWrapper(propDef)
                    if default != propDef:
                        self.__dbg(1,  "changing default w/default")
                        if self._localPropertyDef(id):
                            self.__dbg(1,  "setting def")
                            default=self.manage_editObjectRef(propId=id, propVal=defaultReq)
                            self._setPropertyDef(id, self.rmWrapper(default))
                        else:
                            self.__dbg(1,  "adding def")
                            self.manage_addPropertyDef(id, 'objref', 
                                                 label=self.propertyLabel(id),
                                                 default=defaultReq,
                                                 seq=self.propertySeq(id))
                    elif listSelection != propDef:
                        self.__dbg(1,  "changing default w/list")
                        default=self.manage_editObjectRef(propId=id, propVal=listReq)
                        if self._localPropertyDef(id):
                            self.__dbg(1,  "setting def")
                            self._setPropertyDef(id, self.rmWrapper(default))
                        else:
                            self.__dbg(1,  "adding Def")
                            self._addPropertyDef(id,
                                                 self.rmWrapper(default),
                                                 label=self.propertyLabel(id),
                                                 propType=self.propertyType(id),
                                                 seq=self.propertySeq(id))
                else:
                    self.__dbg(500, "default b4: %s, %s, %s" % (id, default, defaultReq))
                    if propType=='lines':
                        defAliases=REQUEST.get('aliases_'+id, '')
                        if not type(defAliases)==type([]):
                            defAliases=[defAliases]
                        if default!='':
                            if type(default)!=type([]):
                                default=[default]
                            default=default+defAliases
                        else:
                            default=defAliases
                        default=self.removeDuplicates(default)
                        self.__dbg(500, "id %s def: %s" % (id, default))

                    if default != self.propertyDefault(id):
                        self.__dbg( 1, 'Modifying %s, %s' % (self.id, self))
                        ## update default value of property
                        if self._localPropertyDef(id):
                            self.__dbg( 1, 'Setting local property %s to %s ' % (id, default))
                            self._setPropertyDef(id, default)
                        else:
                            self.__dbg( 1, 'Adding property %s w/default %s' % (id, default))
                            self._addPropertyDef(id,
                                                 default,
                                                 label=self.propertyLabel(id),
                                                 propType=self.propertyType(id),
                                                 seq=self.propertySeq(id))
                        ## lets update the keyword list
                        self.modifyKw(id, propType)
                    if propType == 'meme':
                        multiMode=REQUEST.get('multiSelect_'+id, '')
                        self.__dbg(0, "rtned: %s" % self._setMultiMode(id, multiMode))
                        self.__dbg(400, "id %s, default: %s" % (id, self.getProperty(id)))
                            
                ## get label of property from request
                label='label_'+str(id)
                label=REQUEST.get(label, '')
                if label != self.propertyLabel(id):
                    default=self.propertyDefault(id)
                    if self._localPropertyDef(id):
                        self._setPropertyDef(id, default, label=label)
                    else:
                        self._addPropertyDef(id,
                                             default,
                                             label=label,
                                             propType=self.propertyType(id),
                                             seq=self.propertySeq(id))
                        
                ## get sequence number of property from request
                seq='seq_'+str(id)
                seq=float(REQUEST.get(seq, ''))
                if seq != self.propertySeq(id):
                    default=self.propertyDefault(id)
                    ## update sequence number of property
                    if self._localPropertyDef(id):
                        self._setPropertyDef(id, default, seq=seq)
                    else:
                        self._addPropertyDef(id,
                                             default, 
                                             label=self.propertyLabel(id),
                                             propType=self.propertyType(id),
                                             seq=seq)
        return self.manage_definePropertiesForm(self, REQUEST)

    def manage_addPropertyDef(self, id, propType, label='', default='', seq=0, REQUEST=None):
        """Add a new property via the web
        """
        val=default
        if not default: default=self.typeDefault(propType)
        default=self.read_property(default, propType)
        if propType == 'objref':
            self._addPropertyDef(id, None, label, propType, seq)
            if not val: val=self.typeDefault(propType)
            self.__dbg(1,  "val: %s" % (val))
            res=self.manage_editObjectRef(propId=id, propVal=val)
            if res:
                self.__dbg(1,  "res: %s " % res.id)
            else: self.__dbg(1,  "no object added")
            ## store wrapped object
            self._setPropertyDef(id, res)
        else:
            self._addPropertyDef(id, default, label, propType, seq)
            if propType == 'meme':
                self.__dbg(0, "rtned: %s" % self._setMultiMode(id, ''))

        return self.manage_definePropertiesForm(self, REQUEST)

    def manage_delPropertyDefs(self, ids, REQUEST=None):
        """Delete one or more property definitions specified by 'ids'."""
        for id in ids:
            if not self._deleteablePropertyDef(id):
                return MessageDialog(
                    title='Cannot delete %s' % id,
                    message='The property <em>%s</em> cannot be deleted.' % id,
                    action='manage_definePropertiesForm')
            if self.propertyType(id)=='objref':
                self.__dbg(1,  "deleting in delPropertyDefs: %s, in %s"%(id, self))
                self.dump()
                self.manage_editObjectRef(propId=id, propVal=None)
                if self.meta_type in ('CARS Folder', 'CARS Share'):
                    self._delPropertyDef(id)
                self.dump()
            else:
                self.__dbg(20, "deleting property def: %s" %(id))
                propType=self.propertyType(id)
                self._delPropertyDef(id)
                self.deleteMeme(id)
                ## lets update the keyword list
                self.modifyKw(id, propType)
        if REQUEST is not None:
            return self.manage_definePropertiesForm(self, REQUEST)

    def manage_toggleInherited(self, REQUEST=None):
        """Toggle inheritance"""
        self._inherited=not self._inherited
        self.updateAllKws()
        if REQUEST:
            return self.manage_definePropertiesForm(self, REQUEST)

    def cleanObjRefs(self):
        for prop in self.propertyMap():
            self.__dbg(5, "prop: %s" % (prop))
            if self.propertyType(prop['id'])=='objref':
                self.delProperty(prop['id'])
                self._delPropertyDef(prop['id'])

