import string
from PropertyBase import PropertyBase
from Globals import HTMLFile, MessageDialog
from urllib import quote
from string import split, join
from Products.CARS import Log
from AccessControl import getSecurityManager

class PropertyStore(PropertyBase):
    _reqProperty=""

    __dbgLevelDiagnostic = 0
    __className='PropStore'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)
            
    def getProperty(self, id, d=None):
        if self.hasProperty(id):
            return getattr(self.aq_base, id, self.propertyDefault(id))
        return d

    def getPropertyWOdefault(self, id, d=None):
        if self.hasProperty(id):
            return getattr(self.aq_base, id, None)
        return d

    def setProperty(self, id, value):
        if not self.hasProperty(id):
            raise 'Bad Request', 'The property %s does not exist' % id
        self._setPropValue(id, value)

    def delProperty(self, id):
        if not self.hasProperty(id):
            self.__dbg(1, "DEL prop not found: %s" % id)
            return None
        self.__dbg(1, "DELeteing Prop: %s" % id)
        self._delPropValue(id)

    def _setPropValue(self, id, value):
        self._wrapperCheck(value)
        setattr(self, id, value)

    def _delPropValue(self, id):
        try:
            delattr(self,id)
        except: pass

    def manage_editProperties(self, REQUEST):
        """Edit object properties via the web."""
        self.testImmutable()
        for prop in self.propertyMap():
            if 'w' in prop.get('mode', 'wd'):
                id=prop['id']
                propType=self.propertyType(id)
                val=REQUEST.get(id, '')
                value=self.read_property(val, propType, id=id)
                if propType=='objref':
                    listReq=REQUEST.get('list_'+id, '')
                    listSelection=self.read_property(listReq, propType, id=id)
                    propV=self.rmWrapper(self.getProperty(id))
                    if value != propV:
                        default=self.manage_editObjectRef(propId=id, propVal=val)
                    elif listSelection != propV:
                        default=self.manage_editObjectRef(propId=id, propVal=listReq)
                else:
                    self.__dbg(100, "val: %s" % (val))
                    if propType=='lines':
                        valAliases=REQUEST.get('aliases_'+id, '')
                        if not type(valAliases)==type([]):
                            valAliases=[valAliases]
                        self.__dbg(110, "aliases_ %s, value: %s" % (valAliases, value))
                        value=value+valAliases
                        value=self.removeDuplicates(value)
                        if val=='':
                            ## if the default value is blank we must be able
                            ## to insert words from a list without deleting
                            ## the property
                            val=string.join(valAliases, ' ')
                    if val == '':
                        self.__dbg(100, "deleting Propid %s " % id)
                        self.delProperty(id)
                        self.modifyKw(id, propType)
                    elif not self.isPropertyContainerDefault(id, value):
                        self.__dbg(100, "property and default differ will save propid %s, %s" % (id, value))
                        self.setProperty(id, value=value)
                        self.modifyKw(id, propType)
                    elif propType=='meme':
                        ## needs to be treated differently
                        self.setProperty(id, value=value)
                    self.__dbg(200, "property %s set to %s" % (id, value))
        self.updateAllKws()
        self.informContacts("Properties modified", \
                            "Properties for '%s' have been modified" % self.id)
        return MessageDialog(
               title  ='Success!',
               message='Your changes have been saved',
               action ='manage_main')

    def setImmutable(self):
        """Set object to immutable.
        """
        ## copy property values in object so that they cannot be changed by
        ## inheriting default values.
        for prop in self.propertyMap():
            if 'w' in prop.get('mode', 'wd'):
                id=prop['id']
                propType=self.propertyType(id)
                val=self.getProperty(id, '')
                value=self.read_property(val, propType, id=id)
                if propType=='objref':
                    self.__dbg(5, "Object Value: %s, %s" % (val, value))
                    self.manage_editObjectRef(propId=id, propVal=val.getURL())
                else:
                    self.setProperty(id, value=value)
        self._isImmutable=1

    def isImmutable(self):
        if hasattr(self, '_isImmutable'):
            return self._isImmutable
        return None

    def testImmutable(self):
        """Produce an error if object is immutable"""
        if self.isImmutable():
            raise 'Bad Request', 'Object is immutable!'

    def setReqProp(self, prop):
	self._reqProperty=prop

    def getReqProp(self):
	return self._reqProperty

    def memeItemSelected(self, id, idx):
        '''Return true if the item is selected'''
        self.__dbg(2000, "idx: %s, getProp %s" % (idx, self.getProperty(id)))
#        if idx in self.getProperty(id):
        if idx == self.getProperty(id):
            return 1
        return 0

    def getMemeItems(self, id):
        '''Return a list of meme items for an object'''
        r=[]
        memeItems=self.findParentMemes(id, dict=1)
        for idx in self.getProperty(id):
            if memeItems.has_key(idx):
                r.append(memeItems[idx])
        return r








