__doc__ = """externFS: External File Storage class."""

###############################################################################
#
# Copyright (c) 2001 International Electronics and Engineering(IEE),
# Luxembourg (www.iee.lu).
# Written by Dennis Connolly (Dennis.Connolly@iee.lu)
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# Disclaimer
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  In accordance with the license provided for by the software upon
#  which some of the source code has been derived or used, the following
#  acknowledgement is hereby provided :
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
# Copyright (c) 2000 Gregor Heine and Communications & Design Consultancy GmbH,
# Solmsstrasse 2, 60486 Frankfurt, Germany (http://www.cdc-group.com)
# All rights reserved. Written by Gregor Heine <heine@cdc-group.com>
# ExtFile Home: http://www.zope.org/Members/MacGregor/ExtFile/index_html
#
#######################################################################              

__version__ = '0.1'
import Globals
from Globals import HTMLFile, MessageDialog, Persistent
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport import FindSupport
from OFS.content_types import guess_content_type
from webdav.common import rfc1123_date
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
from __main__ import *
import Zope
from Products.ZCatalog.CatalogAwareness import CatalogAware
from DateTime import DateTime
from types import StringType
import urllib, os, string
try: from cStringIO import StringIO
except: from StringIO import StringIO
from Products.CARS import Log

class externFS:
    """Create an external file on the local file system"""
    __dbgLevelDiagnostic = 0
    __className='externFS'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    _externFSbasePath='var/CARS.Repository/'

    def _path(self, urlPath):
        self.__dbg(500, "urlPath: %s" % (os.path.normpath(\
            self._externFSbasePath+urlPath+'/'+self.subPath)))
        return os.path.normpath(self._externFSbasePath+urlPath+'/'+self.subPath)

    def _genNewFn(self, filename, urlPath):
        time=DateTime().strftime('%y%m%d%H%M%S')
        file=os.path.join(self._path(urlPath),"%s_%s"%(time, filename))
        while os.path.isfile(file) or os.path.isfile(file+'%deleted'):
               time=time+1
               file=os.path.join(self._path(urlPath),"%s_%s" % (time, filename))
        return ("%s_%s" % (time, filename))

    def __init__(self, file, filename, title='', \
                 content_type='', subPath='', urlPath=''): 
        """initialize a new instance of externFS"""
        self.content_type=content_type
        self.subPath=subPath
        self.__dbg(500, "urlPath: %s" % (urlPath))
        if os.path.exists(self._path(urlPath))!=1:
                mkdirExc='Cannot create directory: '
                try:
                        os.makedirs(self._path(urlPath))
                except:
                        raise mkdirExc, self._path(urlPath)
        self.id = self._genNewFn(filename, urlPath)
        self.filename=filename
        self.title=title
        self.__dbg(200, "file: %s, %s, %s" % (self.id, self.filename, type(file)))

        cantWriteExc = 'Cannot write: '
        self.__dbg(1, "Attempting to create: %s " % (self.getFullPath(urlPath)))
        fp=open(self.getFullPath(urlPath), 'wb')
        self.rawsize=self._copy_data(file, fp)
        fp.close()
        self.__dbg(100, "Created file %s, contentType: %s" %(self.id, self.content_type))
        self.size = self._bytetostring(self.rawsize)
        self.__dbg(100, "raw %s, size %s" % (self.rawsize, self.size))
        if type(file) is not type(''):
            self.content_type=self._get_content_type(file,file.read(100), \
                                                     self.id,content_type)
            self.__dbg(100, "contentType: %s" % (self.content_type))
        self.__dbg(100, "returning")

    def copyFile(self, infile, outfile, urlPath=''):
        try:
            inFS=open(infile.getFullPath(urlPath), 'rb')
        except IOError:
            raise IOError, "Cannot open %s" % (self.id)
        self._copy_data(inFS, outfile)
        return outfile

    def getFilename(self):
        """Return filename"""
        return self.id

    def getFullPath(self, urlPath=''):
        """Return a full path to the object"""
        return os.path.join(self._path(urlPath), self.id)

    def reconstructExternFSpath(self, oldPath, newPath):
        """Modify the external FS path to the new"""
        self.__dbg(500, "oldPath: %s, newPath: %s" % (oldPath, newPath))
        if oldPath!='' and \
           os.path.exists(self._externFSbasePath+oldPath) and \
           not os.path.exists(self._externFSbasePath+newPath):
            self.__dbg(501, "Renaming folders")
            os.renames(self._externFSbasePath+oldPath, self._externFSbasePath+newPath)
            return
        if os.path.exists(self._externFSbasePath+newPath):
            raise "OS Filesystem", "Path '%s' already exists on server!" % \
                  (self._externFSbasePath+newPath)
            
    def destructExternFSpath(self, urlPath):
        if os.path.exists(self._externFSbasePath+urlPath) and \
           not os.path.exists(self._externFSbasePath+urlPath+'%deleted'):
            os.renames(self._externFSbasePath+urlPath, \
                       self._externFSbasePath+urlPath+'%deleted')
            return urlPath+'%deleted'
        self.__dbg(600, "locating: %s" % (urlPath))
        self.__dbg(601, "does it exist: %s " % (os.path.exists(self._externFSbasePath+urlPath)))
        return urlPath
    
    def getFileId(self):
        """Get file id"""
        return self.id

    def getIcon(self, app):
        """Get icon"""
        # MIME-category
        cat=self.content_type[:string.find(self.content_type, '/')]
        # sub-category 
        sub=self.content_type[string.find(self.content_type, '/')+1:]	
        mimePath='/images/mimeIcons'
        self.__dbg(200, "locating: %s, content Type: %s" % (mimePath, self.content_type))
        found=0
        container=app.unrestrictedTraverse("%s/%s" % (mimePath, cat))
        self.__dbg(200, "found container %s" % (container))
        for ob in container.objectValues():
            self.__dbg(200, "title %s, sub %s" % (ob.title, sub))
            if ob.title==sub:
                self.__dbg(200, "match found: title==sub")
                return ob.absolute_url()
        try:
            return app.unrestrictedTraverse("%s/%s/default.gif" % \
                                            (mimePath, cat)).absolute_url()
        except:
            try: return app.unrestrictedTraverse("%s/default.gif" % \
                                                 (mimePath)).absolute_url()
            except:
                return None
        return
    
    def delete(self, urlPath):
        """Delete file on local file system."""
        fn=self.getFullPath(urlPath)
        try:
            os.rename(fn, fn+'%deleted')
            self.oldFilename=fn
            self.__dbg(1, "Deleted file %s" %(fn))
        except:
            Log.carslog(self, 'Warning: Could not delete (%s)' % fn)

    def undelete(self, newUrlPath):
        """Undelete"""
        self.__dbg(500, "undeleting in urlPath: %s" % (newUrlPath))
        nfn=self.getFullPath(newUrlPath)
        try:
            os.renames(nfn+'%deleted', nfn)
        except:
            Log.carslog(self, 'Warning: Could not undelete (%s)' % nfn+'%deleted')
        self.__dbg(1, "file %s undeleted and located in %s" % (self.filename, \
                                                               self._path(newUrlPath)))

    def get_size(self):
            """Returns the size of the file or image."""
            return self.rawsize

    def getContentType(self):
            """Get the content type of a file or image.
            Returns the content type (MIME type) of a file or image."""
            return self.content_type

    def _get_content_type(self, file, body, id, content_type=None):
            """determine the mime-type"""
            headers=getattr(file, 'headers', None)
            if type(body) is not type(''): body=body.data
            content_type,enc=guess_content_type(getattr(file,'filename',id),body,content_type)
            return content_type

    def _copy_data(self, instream, outstream):
            """Read binary data."""
            if type(instream) is StringType: 
                self.__dbg(100, "writing to stream")
                outstream.write(instream)
                self.__dbg(100, "wrote to stream, %s" % (len(instream)))
                return len(instream)
            self.__dbg(100, "seeking")
            instream.seek(0,2)
            self.__dbg(100, "telling")
            size=instream.tell()		# determine the filesize in bytes
            instream.seek(0)
            blocksize=2<<16
            pos=0
            while pos<size:
                self.__dbg(100, "pos/size: %s/%s" % (pos, size))
                outstream.write(instream.read(blocksize))
                pos=pos+blocksize
            instream.seek(0)
            return size

    def _copyfile (self, filename, outstream):
            """read binary data from file filename"""
            try:
                    instream=open(filename, 'rb')
            except IOError:
                    self._undo()
                    try:
                            instream=open(filename, 'rb')
                    except IOError:
                            raise IOError, ("%s (%s)" %(self.id, filename))
            try:
                    size=self._copy_data(instream, outstream)
                    instream.close()
            except IOError:
                    raise IOError, ("%s (%s)" %(self.id, filename))
            return size

    def _bytetostring (self, value):
            """Convert an int-value (file-size in bytes) to an String
               with the file-size in Byte, KB or MB
            """
            bytes=float(value)
            if bytes>=1000:
                    bytes=bytes/1024
                    if bytes>=1000:
                            bytes=bytes/1024
                            typ=' MB'
                    else:
                            typ=' KB'
            else:
                    typ=' Bytes'
            strg='%4.2f'%bytes
            strg=strg[:4]
            if strg[3]=='.': strg=strg[:3]
            strg=strg+typ
            return strg


	

