__doc__ = """kwLists: maintain keyword lists."""
__version__ = '0.1'
import string
from Products.CARS import Log
from Globals import HTMLFile

class kwLists:
    """Keyword list maintenance"""
    __dbgLevelDiagnostic=0
    __className='kwLists'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    def initKwVars(self):
        self.__generalKws=[]
        self.__kwList=[]
        self.__aliasList=[]
        self.__memeList={}

    manage_aliasesForm=HTMLFile('aliases', globals())
    manage_kwsForm=HTMLFile('kws', globals())
    manage_memeForm=HTMLFile('meme', globals())
    
    def removeDuplicates(self, l):
        ## return a list with duplicate items removed
        def _filter(sList, item):
            if item is None:
                return sList
            if type(item)==type(''):
                item = string.strip(item)
            if item not in sList and item!='':
                sList.append(item)
            return sList
        ## little hack to allow _filter to append to a blank list
        ## e.g. l at this point is [2,1,4,1,2,5...] we need [[],2,1,4,1,2,5...]
        ## but first lets copy the original list, so we don't change its current
        ## contents
        l=l[:]
        l.insert(0, [])
        self.__dbg(500, "rmDuplicates l %s" % (l))
        return reduce(_filter, l)

    def _resetList(self, list, id):
        seq=self.findRef(list, id)
        self.__dbg(20, "list: %s" % (list))
        if seq is not None:
            # reference found
            list[seq]=tuple((id, []))
            self.__dbg(20, "reset list: %s" % (list))
        self._p_changed=1

    def _getKwList(self):
        return self.__kwList

    def _setKwList(self, l):
        self.__kwList=l
        
    def resetKwList(self, id):
        self._resetList(self._getKwList(), id)

    def findRef(self, list, ref):
        ## return sequence in list that matches ref
        def filterRef(ref, kws, seq):
            r, w=kws
            if ref==r:
                return seq

        s=filter(lambda seq, filterRef=filterRef, ref=ref, list=list: \
                 filterRef(ref, list[seq], seq) is not None, range(len(list)))
        self.__dbg(20, "sequence: %s" % (s))
        if len(s)>0:
            return s[-1]
        return None

    def getKwList(self, ref):
        self.__dbg(200, "finding %s in __kwList %s" % (ref, self._getKwList()))
        seq=self.findRef(self._getKwList(), ref)
        if seq is not None:
            r,l=self._getKwList()[seq]
            return l
        return []

    def _registerKw(self, kwList, ref, wList):
        seq=self.findRef(kwList, ref)
        if seq is not None:
            # insert new words into existing list
            r,l=kwList[seq]
            # find which words are unique to the current list
            nList=filter(lambda w, l=l: w not in l, wList)
            self.__dbg(5, "nlist: %s" % (nList))
            kwList[seq]=tuple((ref, l+nList))
        else:
            # brand new keyword, just append to the list
            self.__dbg(5, "appending: %s" % (wList))
            kwList.append(tuple((ref, wList)))
                                                                          
    def registerKw(self, ref, wList=[]):
        if wList is None:
            return
        self.__dbg(2000, "before: %s, %s" % (self.getKwList(ref), wList))
        # make a copy of the input word list or data will get corrupted
        # in the original list
        wordList=wList
        wordList=self.removeDuplicates(wordList)
        # required to copy the list otherwise zope will not commit
        # the modified list to the database on the next restart
        p=self._getKwList()[:]
        self._registerKw(p, ref, wordList)
        self._setKwList(p)
        self.__dbg(2, "after: %s" % self.getKwList(ref))
        self.__dbg(2, "__kwList: %s" % (self._getKwList()))

    def findAliases(self, aliases, kwList):
        def filterKw(self, seqList):
            ## remove duplicate sequence numbers and return a list of unique sequences
            l=map(lambda s: s[-1], seqList)
            return self.removeDuplicates(l)
        def findKw(aliases, k, s):
            ## return the sequence number if a keyword is found
            if k in aliases[s]:
                return s
        if type(kwList)!=type([]):
            kwList=[kwList]
        kwList=self.removeDuplicates(kwList)
        self.__dbg(100, "finding aliases for kws: %s in aliases: %s" % (kwList, aliases))

        ## determine which keyword is in a list by return the sequence no. for that list
        s1=map(lambda k, aliases=aliases, findKw=findKw: \
               filter(lambda s, k=k, aliases=aliases, findKw=findKw: \
                      findKw(aliases,k,s)!=None, range(len(aliases))), kwList)
        ## filter blank sequence elements
        s2=filter(lambda s: s!=[], s1)

        ## generate an aliase keyword list from a list of sequence numbers
        aliasList=[]
        for x in filterKw(self, s2):
            map(lambda kw, aliasList=aliasList: aliasList.append(kw), aliases[x])

        ## find the keywords that do not have any aliases
        freeList=filter(lambda kw, aliasList=aliasList: kw not in aliasList, kwList)
        if type(freeList)!=type([]):
            if freeList=='':
                freeList=[]
            else:
                freeList=[freeList]
        ## combine list of aliases and non alias keywords
        combinedList=aliasList+freeList
        self.__dbg(30, "returning %s " % (combinedList))
        return combinedList

    def rtnAliasList(self, idx=None):
        if idx:
            return self._getAliasList()[idx]
        return self._getAliasList()

    def _getAliasList(self):
        return self.__aliasList

    def _setAliasList(self, l):
        self.__aliasList=l
        
    def validateAliasList(self, alias=None, REQUEST=None):
        if alias:
            alias=self.removeDuplicates(alias)
        if REQUEST is not None:
            searchList=[]
        else:
            searchList=self._getAliasList()
        for x in range(len(self._getAliasList())):
            if REQUEST is not None:
                print "REQUEST", REQUEST.get(str(x))
                alias=string.split(REQUEST.get(str(x)), '\n')
            self.__dbg(20, "alias: %s" % (alias))
            alias=self.removeDuplicates(alias)
            if alias:
                if len(alias) < 2:
                    raise "Alias Error", "More than one word needed to make an alias group"
                duplicateWords=[]
                for l in searchList:
                    fl=filter(lambda w, alias=alias: w in alias, l)
                    if fl:
                        duplicateWords.append(fl)
                self.__dbg(20, "duplicateWords: %s" % duplicateWords)               
                if duplicateWords:
                    raise "Alias Error", "Duplicate words in aliases are prohibited: %s" \
                          % ( duplicateWords )
                if REQUEST is not None:
                    searchList.append(alias)
        if REQUEST is not None:
            return searchList
        ## the alias must be returned and appended with this value
        return alias
    
    def manage_editAlias(self, REQUEST):
        """Edit aliases"""
        self._setAliasList(self.validateAliasList(REQUEST=REQUEST))
        return self.manage_aliasesForm(self, REQUEST)

    def manage_addAlias(self, REQUEST):
        """Adding alias"""
        alias=REQUEST['alias']
        if alias:
            alias=list(string.split(alias,'\n'))
            self.__dbg(16, "alias: %s" % (alias))
            ## must append the returned alias otherwise 'alias' in this namespace
            ## appears corrupted preventing addition of new aliases
            alias=self.validateAliasList(alias=alias)
            self._getAliasList().append(alias)
            self.__dbg(17, "added %s\n%s" % (alias, self._getAliasList()))
            self._p_changed=1
        return self.manage_aliasesForm(self, REQUEST)

    def manage_aliases(self, REQUEST):
        """Manage aliases"""
        return self.manage_aliasesForm(self, REQUEST)

    def _getMeme(self, id):
        try:
            return self.__memeList[id]
        except:
            return None

    def _deleteMeme(self, id):
        if self.__memeList.has_key(id):
            del self.__memeList[id]

    def deleteMeme(self, id):
        self._deleteMeme(id)
            
    def _addMeme(self, id, ml):
        self._deleteMeme(id)
        self.__memeList[id]=ml
        self._p_changed=1

    def rtnMeme(self, id, dict=0):
        meme=self._getMeme(id)
        r=[]
        if meme:
            idxCnt, ml=meme
            if dict:
                ## return dictionary
                return ml
            self.__dbg(500, "indexCnt: %s" % (idxCnt))
            for idx in ml.keys():
                text=ml[idx]
                self.__dbg(500, "idx %s, text %s" % (idx, text))
                r.append({'idx': idx, 'text': text})
        return r

    def manage_addMeme(self, REQUEST):
        """Add to meme list"""
        for eProp in self.fullPropertyMap():
            if eProp['type']=='meme':
                id=eProp['id']
                al=REQUEST.get('aliases_'+id)
                if al is not None:
                    if type(al)!=type([]):
                        al=[al]
                    self.__dbg(400, "al: %s" % (al))
                    for m in al:
                        meme=self._getMeme(id)
                        self.__dbg(500, "adding: %s, meme: %s" % (m, meme))
                        if meme:
                            idxCnt, ml=meme
                        else:
                            idxCnt=0
                            ml={}
                        duplicate=0
                        for idx in ml.keys():
                            if ml[idx]==m:
                                duplicate=1
                        if not duplicate:
                            ml[str(idxCnt)]=m
                            idxCnt=idxCnt+1
                            self._addMeme(id, (idxCnt, ml))
        return self.manage_memeForm(self, REQUEST)
                                  
    def manage_deleteMeme(self, REQUEST):
        """Remove meme from list"""
        for eProp in self.fullPropertyMap():
            if eProp['type']=='meme':
                id=eProp['id']
                al=REQUEST.get(id+"/items")
                if al is not None:
                    self.__dbg(400, "id: %s, al: %s" % (id, al))
                    if type(al)!=type([]):
                        al=[al]
                    self.__dbg(400, "al: %s" % (al))
                    for m in al:
                        meme=self._getMeme(id)
                        if meme:
                            idxCnt, ml=meme
                            self.__dbg(400, "deleting %s from %s" % (m, ml))
                            del ml[m]
                    self.rtnMeme(id)
        return self.manage_memeForm(self, REQUEST)                

    def manage_memes(self, REQUEST):
        """Manage memes"""
        return self.manage_memeForm(self, REQUEST)

    def _setKws(self, kws):
        self.__generalKws=kws

    def manage_editKws(self, kws, REQUEST):
        """Manage kws"""
        kws=string.split(kws, '\n')
        kws=self.removeDuplicates(kws)
        al=[]
        for e in self.rtnAliasList():
            for f in e:
                al.append(f)
        r=filter(lambda k, al=al: k in al, kws)
        self.__dbg(200, "kws: %s, r: %s al: %s" % (kws, r, al))
        if len(r)>0:
            raise "Duplicate Keywords", "The following keywords exist as aliases: %s" % r
        self._setKws(kws)
        return self.manage_kwsForm(self, REQUEST)

    def rtnGeneralKws(self, rtnList=1):
        if rtnList:
            return self.__generalKws
        return string.join(self.__generalKws, '\n')

    def manage_kws(self, REQUEST):
        """Manage a set of keywords"""
        return self.manage_kwsForm(self, REQUEST)
        
