import ExtensionClass
from AccessControl import getSecurityManager
from string import split, join, replace
from Globals import HTMLFile, MessageDialog
from Products.CARS.sendmail import sendmail
from Products.CARS import Log
import string
from types import StringType
from DateTime import DateTime
from Products.CARS.kwLists import kwLists 
from Products.CARS.externFS import externFS

class objBase(externFS.externFS):
    """Basic object house keeping e.g. pathLists, references to/from object instances etc."""

    confirmDeleteForm=HTMLFile('confirmDelete', globals())
    __dbgLevelDiagnostic = 0
    __className='objBase'
    
    def __dbg(self, level, dbgLine):
        Log.debug( self.__dbgLevelDiagnostic, level, dbgLine, self.__className)

    __delIds=[]
    __cloned=0
    __deleted=0

    def __init__(self):
        self.refObjsCnt_to=0
        self.refObjsCnt_from=0
        self.refFromList=[]
        self.myInstance=self
        self.revisioNum=1
        self.version=0.1
        self.__dbg(10, "Initialised base object")

    def showAttr(self):
        """Show all attributes of object"""
        self.__dbg(2000, "id: %s, %s" % (self.id, len(self)))
        return self.id

    def isCloned(self):
        return self.__cloned
        
    def genId(self):
        '''Generate id based on OSI Standard'''
        return str(DateTime().strftime('%y%m%d%H%M%S'))

    def informContacts(self, subject, message=''):
        """Lets get a list of email contacts to inform them that a new object has been added.
        """
        emailList=self.getCompleteProp('inform')
        body=message + \
              "\n\nTry this link: %s" % self.absolute_url()
        thisUser=getSecurityManager().getUser()
        subject="CARSmessage: %s" % (subject)
        for user in emailList:
            sendmail(user, thisUser, subject, body)

    def currentObjSelected(self, id):
        if self.getProperty(id):
            return self.getProperty(id).getURL()
        return None

    def addToList(self, ob):
        self.refFromList.append(ob)
        self._p_changed=1

    def getFromList(self):
        return self.refFromList

    def getRefObjsCnt_to(self):
	return self.refObjsCnt_to

    def getRefObjsCnt_from(self):
	return self.refObjsCnt_from

    def manage_confirmDelete(self, REQUEST=None):
	"""Continue with delete"""
	self.__delIds=REQUEST.ids
	if REQUEST:
		REQUEST['RESPONSE']=self.confirmDeleteForm(self)
	return self.confirmDeleteForm(self)

    def manage_delCARSobjectsConfirmed(self, REQUEST=None):
	"""delete confirmed"""
        self.__dbg(1, "deleting REFS: %s" % self.__delIds)
	self.deleteObjReferences(ids=self.__delIds)
	r=self.manage_delObjects(self.__delIds, REQUEST=REQUEST)
        self.informContacts("Objects Deleted", \
                            "Object(s) have been deleted %s \
                            in the following location." % self.__delIds)
        return r

    def delCARSobjects(self, REQUEST):
        """Necessary to call this method first to be sure that all values have been
        committed to Zope before returning control back to user"""
        return self.manage_delCARSobjects(REQUEST)
        
    def manage_delCARSobjects(self, REQUEST):
	"""Remove CARS objects only if no links are attached to them"""
	delIds=self.checkObjs(REQUEST.ids)
	if len(REQUEST.ids)!=len(delIds):
	    return self.manage_confirmDelete(REQUEST=REQUEST)
        self.__dbg(1, "delCARSobjs: %s" % ( REQUEST.ids ))
	r=self.manage_delObjects(REQUEST.ids, REQUEST=REQUEST)
        self.updateAllKws()
        self.informContacts("Objects Deleted", \
                            "Object(s) have been deleted %s \
                            in the following location." % REQUEST.ids)
        return r

    def manage_pasteCARSObjects(self, REQUEST):
	"""Paste CARS Objects"""	
        return self.manage_pasteObjects(REQUEST=REQUEST)

    def manage_beforeDelete(self, item, container):
        self.__dbg( 3, "b4Delete: %s, %s" % (item.id, item))
        ## set pathlist of object and deduct reference counters
        self.createPathList(self.absolute_url(1))
        self.updateRefCnt('from', len(self.refFromList) * (-1))
        self.updateRefCnt('to', self.getObjsRefsInPropertyMap() * (-1))
        self.__deleted=1

    def manage_afterClone(self, item):
        self.__dbg( 3, "aftRClone: %s, %s" % (self.id, self))
        self.__dbg( 3, "myInstance: %s, %s" % (self.myInstance, item))
      
        self.refObjsCnt_to=0
        ## we must reset these lists to prevent new instances of objects
        ## from being created.
        self.pathList=[]
        self.refFromList=[]
        self.__cloned=1
    
    def manage_afterAdd(self, item, container):
        self.__dbg( 30, "aftRAdd: %s, %s" % (item.id, item))
        ## set the pathlist of the object to point to the current path
        ## and update reference counters in new path
        try:
            self.__dbg(300, "getURL after Add %s " % (self.getURL(container=1)))
            app=self.getPhysicalRoot()
            ob=app.unrestrictedTraverse(self.getURL(container=1))
            ob.updateAllKws()
        except:
            pass
        if not self.__cloned:
            self.createPathList(self.absolute_url(1))
            if self.__deleted:
                self.updateRefCnt('from', len(self.refFromList))
                self.updateRefCnt('to', self.getObjsRefsInPropertyMap())
        else:
            self.removeAllSubObjects()
            self.__cloned=0
        self.updateAllKws()

    def manage_getReferences(self):
        '''Locate references of object'''
        self.__dbg(2000, "locating references")
        r=[]
        for ob in self.getFromList():
            url=ob.getURL()
            r.append({'url': url, 'link': string.replace(url, ' ', '%20')})
        self.__dbg(2000, "returning: %s" % r)
        return r
            
    def getObjsRefsInPropertyMap(self):
        '''return the number of objects references found in the property map'''
        cnt=0
        for prop in self.propertyMap():
            if prop['type']=='objref':
               propId=prop['id']
               ob=self.getProperty(propId)
               ## make sure that a reference is pointed to
               if ob:
                   self.__dbg(3, "Obj referenced: %s, %s" % (ob.id, ob))
                   cnt=cnt+1
        return cnt
 
    def callObject(self, id, cmd=None):
        ob=self.getProperty(id)
        if ob:
            if hasattr(ob, 'getURL') and cmd=='url':
                return ob.getURL(1)
            if hasattr(ob, 'getURL') and cmd=='urlPath':
                return ob.getURL(0)
            if hasattr(ob, 'title') and cmd=='title':
                if ob.title:
                    return str(ob.title)
                else:
                    cmd='id'
            if hasattr(ob, 'id') and cmd=='id':
                return str(ob.id)
            else: return str(ob)
	return ""

    def manage_editObjectRef(self, propId=None, propVal=None, oldOb=None, REQUEST=None):
	"""Edit object reference to other object via the web."""
        self.testImmutable()
        if REQUEST:
            propId=REQUEST.get('PROPERTYID', '')
            propVal=REQUEST.get('PROPERTY', '')
        if propVal=='None':
            propVal=None
	if propVal:
	    try:
		ob=self.createPathList(propVal)
	    except:
                if REQUEST:
                    return MessageDialog(
                            title  ='Failed!',
                            message="Link to object not found: '%s'" % propVal,
                            action ='manage_propertiesForm')
                raise 'Bad Request', "Link to object not found: '%s'" % propVal
	else: ob=None

        self.__dbg(1, "manage editobjectref self: %s" % self)
        self.addPathList()
        oldOb = self.getPropertyWOdefault(propId)
        if oldOb and not(type(oldOb) is StringType):
            # remove reference to self from object
            self.__dbg(1, "delRef oldOb: %s, %s"%(oldOb, self.refObjsCnt_to))
            self.delReference(oldOb)
        if ob:
            self.__dbg(1, "addRef ob: %s" % ob)
            ob.addReference(self)
            self.setProperty(propId, ob.aq_base)
        else:
            self.__dbg(1, "DELETING prop in manage_editObjs %s" % propId)
            self.delProperty(propId)
        if REQUEST:
            return MessageDialog(
                    title  = 'Success!',
                    message= 'Reference has been set',
                    action = 'manage_main')
        return ob

    def deleteObjReferences(self, ids=[]):
	"""Delete object references"""
	for id in ids:
            ob=self._getOb(id)
            self.__dbg(1, "__Deleting Ref: %s, %s" % (ob.id, ob))
            ## first remove any objects referenced in the propertyMap 
            if ob.getRefObjsCnt_to():
                for prop in ob.propertyMap():
                    if prop['type']=='objref':
                       propId=prop['id']
                       if ob.getProperty(propId):
                           ob.delReference(ob.getProperty(propId))
            ## next remove all references pointing to this object
            app=self.getPhysicalRoot()
            self.__dbg(1, "from list: %s" % ob.refFromList)
            for refObj in ob.refFromList[:]:
                self.__dbg(1, "refobj: %s" % refObj)
                obj=app.unrestrictedTraverse(refObj.getURL())
                #remove object(refObj) from property map of obj
                obj.removeRefsFromPropertyMap(ob)
            if getattr(ob, 'isPrincipiaFolderish', None) and \
                hasattr(ob.aq_base, 'subObjects'):
		   if ob.getRefObjsCnt_from() or ob.getRefObjsCnt_to():
			ob.deleteObjReferences(ids=ob.objectIds())
		      
    def removeRefsFromPropertyMap(self,ob=None, id=None):
        if self.meta_type in ('CARS Share', 'CARS Folder'):
            delDef=1
        else: delDef=0
        self.__dbg(1, "current id: %s" % id)
        self.__dbg(1, "current ob: %s, %s" % (ob.id, ob))
        if id is None: self.__dbg(1, "id is none")
	for prop in self.propertyMap():
	    if 'w' in prop.get('mode', 'wd'):
		if prop['type'] == 'objref':
		   propId=prop['id']
                   propDef=self.propertyDefault(propId)
		   if self.rmWrapper(self.getProperty(propId))==self.rmWrapper(ob) and \
                      ((id is None) or (id == propId)):
                        self.__dbg(1, "ok so far. about to delete reference")
                        self.__dbg(1, "prop is %s" % self.getProperty(propId))
                        self.__dbg(1, "prop Def: %s" % self.propertyDefault(propId))
                        self.__dbg(1, "self meta_type: %s" % self.meta_type)
                        if self.meta_type in ('CARS Folder', 'CARS Share'):
                            self.__dbg(1, "deleting propertyDef")
                            self.dump()
                            self.delReference(self.getProperty(propId))
                            self.__dbg(1, "PropertyId: %s" % (propId))
                            self.delProperty(propId)
                            if self.meta_type == 'CARS Folder':
                                self._delPropertyDef(propId)
                            else:
                                self.__dbg(200, "self.id: %s, self: %s" % (self.id, self))
                                self._setPropertyDef(propId,
                                                     None,
                                                     label=self.propertyLabel(propId),
                                                     propType=self.propertyType(propId),
                                                     seq=self.propertySeq(propId))
                        else:
                            self.__dbg(1, "deleting property")
                            self.delReference(self.getProperty(propId))
                            self.delProperty(propId)
                        self.__dbg(1, "prop after is %s" % self.getProperty(propId))
                        self.__dbg(1, "prop Def: %s" % self.propertyDefault(propId))

    def getURL(self, url=None, container=0):
	"""get the url of a referenced object"""
	objUrlList=['']
        if container:
            pathList=self.pathList[:-1]
        else:
            pathList=self.pathList
	try:
	    for x in self.pathList:
		objUrlList.append(x.id)
	except: pass
        if url:
            return replace(join(objUrlList, '/'), ' ', '%20')
        return join(objUrlList, '/')

    def getCurrentPathList(self):
	"""Return a path list of object instances"""
        app=self.getPhysicalRoot()
	pathList=[]
	path=''
	for x in self.getPhysicalPath()[1:]:
	    path=path+'/'+x
	    pathList.append(app.unrestrictedTraverse(path))
	return pathList

    def createPathList(self, path2Obj):
	"""create a path list for the given object, returning reference to object"""
        app=self.getPhysicalRoot()
	ob=app.unrestrictedTraverse(string.replace(path2Obj, "%20", " "))
	ob.pathList=[]
	path=''
	for x in ob.getPhysicalPath()[1:]:
	    path=path+'/'+x
	    ob.pathList.append(app.unrestrictedTraverse(path))
	return ob

    def addPathList(self, ids=None):
        """Add path list to objects"""
	curPathList=self.getCurrentPathList()
        if ids:
	   for id in ids:
		ob=self._getOb(id)
		ob.pathList=curPathList[:]
		ob.pathList.append(ob)
                self.__dbg(1, "ob.pathList: %s" % ob.pathList)

	else:
	   self.pathList=curPathList

    def updatePath(self):
	"""Update path of all objects in current container"""
	oldPathList=None
	curPathList=self.getCurrentPathList()
 	cp=getSecurityManager().checkPermission
	for id in self.objectIds():
	    ob=self._getOb(id)
       	    if cp('Access contents information', ob) and hasattr(ob, 'pathList'):
	    	if ob.pathList and (ob.pathList!=curPathList):
		   oldPathList=ob.pathList[:]
                   if getattr(ob, 'isPrincipiaFolderish', None) and \
                      hasattr(ob.aq_base, 'subObjects'):
			ob.updatePath()
		   ob.pathList=curPathList[:]
	  	   ob.pathList.append(ob)
	return oldPathList

    def updatePathList(self, RESPONSE=None):
	"""Update path list of objects"""
        oldPathList=self.updatePath()
        oldContainerObj=oldPathList[-2]
        #the order is significant. Rebuild from old container first
        oldContainerObj.rebuildRefCnt()
        self.rebuildRefCnt()
	if RESPONSE:
		return RESPONSE
	return

    def rebuildRefCnt(self):
	"""rebuild reference count in container structure"""
	r=self.tallyRefCnt()
	self.refObjsCnt_to=r['total_RefCnt_to']
	self.refObjsCnt_from=r['total_RefCnt_from']
	if hasattr(self, 'aq_parent') and hasattr(self.aq_parent, 'rebuildRefCnt'):
		self.aq_parent.rebuildRefCnt()

    def tallyRefCnt(self):
	"""Tally reference count in current container"""
	total_RefCnt_to=0
	total_RefCnt_from=0
	for id in self.objectIds():
	    ob=self._getOb(id)
	    try:
	       total_RefCnt_to=total_RefCnt_to+ob.refObjsCnt_to
	    except: pass
	    try:
	       total_RefCnt_from=total_RefCnt_from+ob.refObjsCnt_from
	    except: pass
	return {'total_RefCnt_to': total_RefCnt_to, 'total_RefCnt_from': total_RefCnt_from}

    def adjustRefObjsCnt(self, opt, val):
	if not opt in ['to', 'from']:
		raise 'BadRequest', 'Contact IT. Variable opt(%s) out of range in objBase.incRefObjsCnt()' % opt

	if opt=='to':
  	   if self.getRefObjsCnt_to():
		self.refObjsCnt_to=self.refObjsCnt_to+val
		if self.refObjsCnt_to<0:
		   self.refObjsCnt_to=0
	   else:	
		if val==-1:
		   self.refObjsCnt_to=0
		else:
                   self.__dbg(1, "self is %s" % self)
                   self.__dbg(1, "setting 'to' val to: %s" % val)
		   self.refObjsCnt_to=val

	if opt=='from':
	   if self.getRefObjsCnt_from():
		self.refObjsCnt_from=self.refObjsCnt_from+val
		if self.refObjsCnt_from<0:
		   self.refObjsCnt_from=0
	   else:
		if val==-1:
		   self.refObjsCnt_from=0
		else:
		   self.refObjsCnt_from=val
	
    def updateRefCnt(self, opt, val):
	"""Propagate up through hierarchy and update reference variable"""
        if val!=0:
            for ob in self.pathList:	
                ob.adjustRefObjsCnt(opt, val)

    def addReference(self, ob):
	"""Modify references to/from object(ob)"""
	#update refererence counters.  Needed if removing objects from zope
	self.refFromList.append(ob.aq_base)
        self._p_changed=1
	self.updateRefCnt('from', 1)
	ob.updateRefCnt('to', 1)
        self.__dbg(3, "%s refFromList: %s" % (self.id, self.refFromList))

    def delReference(self, ob):
	"""Remove reference to object(self)"""
        try:
            ob.refFromList.remove(self.aq_base)
            ob._p_changed=1
            self.updateRefCnt('to', -1)
            ob.updateRefCnt('from', -1)
        except: self.__dbg(1, "ERROR: trying to remove %s from %s, %s" % (self.id, ob.id, ob))

    def checkObjs(self, ids):
	"""Check if there exists any references to objects within id list"""
	delIds=[]
	for id in ids:
	    ok=1
	    ob=self._getOb(id)
	    try:
	        if ob.refObjsCnt_from!=0:
		   ok=0
	    except: pass
	    try:
		if ob.refObjsCnt_to!=0:
		   ok=0
	    except: pass
	    if ok:
		delIds.append(id)
	    else:
		break
	return delIds

    def displayCnt(self):
        s=''
        try:
           s=s+str(self.refObjsCnt_to)
        except: s=s+"0"
        try:
           s=s+"/"+str(self.refObjsCnt_from)
        except: s=s+"/0"
        return "%s: %s" % (self.id, s)





































